# -*- coding: utf-8 -*-

'''
Copyright (C) 2015-2018 Team C All Rights Reserved

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

bl_info = {
    "name": "BoxCutter",
    "description": "Box Cutter 6.9.0: Poly Dirk",
    "author": "AR, MX, JL, EA, proxe, dairin1d, AgentX, RedFrost",
    "version": (0, 6, 9, 0),
    "blender": (2, 79, 0),
    "location": "View3D",
    # "warning": "Experimental Box Cutter",
    "wiki_url": "http://masterxeon1001.com/2018/09/25/boxcutter-6-9-0-poly-dirk/",
    "category": "Object"}

import bpy
import os
from bpy.props import PointerProperty
from . import developer_utils
from . material import BoxCutterMaterialOptions

modules = developer_utils.setup_addon_modules(__path__, __name__, "bpy" in locals())
addon_keymaps = []


def register_qkeymap():
    addon = bpy.context.window_manager.keyconfigs.addon
    km = addon.keymaps.new(name="3D View", space_type="VIEW_3D")

    kmi = km.keymap_items.new('wm.call_menu', 'E', 'PRESS', ctrl=False, shift=False, alt=True)
    kmi.properties.name = "boxcutter_menu"
    addon_keymaps.append((km, kmi))

    kmi = km.keymap_items.new('wm.call_menu_pie', 'A', 'PRESS', ctrl=False, shift=False, alt=True)
    kmi.properties.name = "boxcutter_pie_menu"
    addon_keymaps.append((km, kmi))


def unregister_qkeymap():
    wm = bpy.context.window_manager
    for km in addon_keymaps:
        for kmi in km.keymap_items:
            km.keymap_items.remove(kmi)
        wm.keyconfigs.addon.keymaps.remove(km)
    addon_keymaps.clear()


def register():
    bpy.utils.register_module(__name__)
    bpy.types.WindowManager.BoxCutter_material_options = PointerProperty(type=BoxCutterMaterialOptions)

    kc = bpy.context.window_manager.keyconfigs.addon

    # create the mode switch menu hotkey
    km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')

    kmi = km.keymap_items.new('boxcutter.draw_boolean_layout', 'W', 'PRESS', ctrl=False, shift=False, alt=True)
    kmi.active = True
    addon_keymaps.append((km, kmi))

    kmi = km.keymap_items.new('boxcutter.switch', 'TWO', 'PRESS', ctrl=False, shift=False, alt=True)
    kmi.active = True
    addon_keymaps.append((km, kmi))


def unregister():
    bpy.utils.unregister_module(__name__)
    del bpy.types.WindowManager.BoxCutter_material_options

    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()
