import bpy
import blf
from bgl import *
import math
from .. draw_px import draw_button_text
from ... preferences import preference
from ... utils.view_3d import scale
from ... utils.space_3d import point_to_2d


def draw_extending_circle(self, context):

    if preference().BoxCutter_shape_moving:
        center = self.mouse_path[0] + self.savedx, self.mouse_path[1] + self.savedy
    else:
        if self.center is "MOUSE":
            center = self.first_mouse_x, self.first_mouse_y
        elif self.center is "CURSOR":
            xc, yc, zc = self.cursor3d_pos
            center = point_to_2d(self, context, xc, yc, zc)

    if self.mode == "SLICE":
        bgR, bgG, bgB, bgA = preference().BC_yellowbox_color
        glColor4f(bgR, bgG, bgB, bgA)
    elif self.mode == "CUT":
        bgR, bgG, bgB, bgA = preference().BC_redbox_color
        glColor4f(bgR, bgG, bgB, bgA)
    elif self.mode == "MAKE":
        bgR, bgG, bgB, bgA = preference().BC_graybox_color
        glColor4f(bgR, bgG, bgB, bgA)
    elif self.mode == "PANEL":
        bgR, bgG, bgB, bgA = preference().BC_purplebox_color
        glColor4f(bgR, bgG, bgB, bgA)
    elif self.mode == "KNIFE":
        bgR, bgG, bgB, bgA = preference().BC_bluebox_color
        glColor4f(bgR, bgG, bgB, bgA)

    radius = abs(math.hypot(self.mouse_path[0] - center[0], self.mouse_path[1] - center[1]))
    amount = self.n

    self.list_dx, self.list_dy = calc_circle_pints(center, radius, amount)

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)
    glBegin(GL_TRIANGLE_FAN)

    for x, y in zip(self.list_dx, self.list_dy):
        glVertex2f(x, y)

    glEnd()

    if preference().BC_use_mirror:
        if not self.clump_mirror:
            if preference().BoxCutter_mirror_axis_x:
                    draw_mirror(self, context, "X")
            if preference().BoxCutter_mirror_axis_y:
                    draw_mirror(self, context, "Y")
            if preference().BoxCutter_mirror_axis_x and preference().BoxCutter_mirror_axis_y:
                    draw_mirror(self, context, "XY")
    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)
    if preference().BoxCutter_shape_moving is True:
        center = self.mouse_path[0] + self.savedx, self.mouse_path[1] + self.savedy
        draw_button_text(True, center[0], center[1], '{}' .format(preference().BoxCutter_circle_vert_count), preference().BoxCutter_display_text_size)
    else:
        draw_button_text(True, self.first_mouse_x, self.first_mouse_y, '{}' .format(preference().BoxCutter_circle_vert_count), preference().BoxCutter_display_text_size)

    glPointSize(preference().BoxCutter_verts_size)
    bgR, bgG, bgB, bgA = preference().BC_verts_color
    glColor4f(bgR, bgG, bgB, bgA)
    glBegin(GL_POINTS)
    for x, y in zip(self.list_dx, self.list_dy):
        glVertex2f(x, y)
    glEnd()


def calc_circle_pints(center, radius, amount):
    list_dx = []
    list_dy = []

    x_offset, y_offset = center
    angle = math.radians(360 / amount)
    for i in range(amount):
        list_dx.append(math.cos(i*angle) * radius + x_offset)
        list_dy.append(math.sin(i*angle) * radius + y_offset)

    return list_dx, list_dy


def draw_mirror(self, context, axis):

    object = bpy.context.active_object
    if object is not None:
        if preference().BoxCutter_mirror_type == "3D":
            origin = point_to_2d(self, context, self.first_active.location[0], self.first_active.location[1], self.first_active.location[2])
        else:
            origin = point_to_2d(self, context, bpy.context.space_data.cursor_location[0], bpy.context.space_data.cursor_location[1], bpy.context.space_data.cursor_location[2])
    else:
        origin = point_to_2d(self, context, 0, 0, 0)

    glBegin(GL_TRIANGLE_FAN)

    for x, y in zip(self.list_dx, self.list_dy):
        point = [x, y]
        mirror = scale(origin, point, axis)
        glVertex2f(mirror[0], mirror[1])
    glEnd()
