import bpy
from bgl import *
import bpy_extras.view3d_utils
from .. draw_px import draw_button_text
from ... preferences import preference
from mathutils import Vector
from mathutils.geometry import tessellate_polygon
from ... utils.view_3d import scale, centroid
from ... utils.space_3d import point_to_2d, region3d
from ... utils.blender_ui import region_dimensions


def draw_extending_curve(self, context):
    region = context.region
    rv3d = region3d(self, context)

    self.last_x_position = self.mouse_path[0]
    self.last_y_position = self.mouse_path[1]

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)

    if self.draw_snap_angle:
        glColor4f(0, 0, 0, 0.5)
        glLineWidth(1)
        guide_len = Vector((region.width, region.height)).magnitude
        guide0 = self.snap_angle_pos # - self.snap_angle_dir * guide_len
        guide1 = self.snap_angle_pos + self.snap_angle_dir * guide_len
        glBegin(GL_LINES)
        glVertex2f(guide0[0], guide0[1])
        glVertex2f(guide1[0], guide1[1])
        glEnd()

    if self.mode == "SLICE":
        bgR, bgG, bgB, bgA = preference().BC_yellowbox_color
        glColor4f(bgR, bgG, bgB, bgA)
    elif self.mode == "CUT":
        bgR, bgG, bgB, bgA = preference().BC_redbox_color
        glColor4f(bgR, bgG, bgB, bgA)
    elif self.mode == "MAKE":
        bgR, bgG, bgB, bgA = preference().BC_graybox_color
        glColor4f(bgR, bgG, bgB, bgA)
    elif self.mode == "PANEL":
        bgR, bgG, bgB, bgA = preference().BC_purplebox_color
        glColor4f(bgR, bgG, bgB, bgA)
    elif self.mode == "KNIFE":
        bgR, bgG, bgB, bgA = preference().BC_bluebox_color
        glColor4f(bgR, bgG, bgB, bgA)

    glPointSize(self.point_radius*2)
    glLineWidth(3)

    if len(self.list_location_3d) == 2:
        glBegin(GL_TRIANGLE_FAN)

        if len(self.list_location_3d) > 1:
            halfsize = 1.5 # line width: 3
            pA = bpy_extras.view3d_utils.location_3d_to_region_2d(region, rv3d, self.list_location_3d[0])
            pB = bpy_extras.view3d_utils.location_3d_to_region_2d(region, rv3d, self.list_location_3d[1])

        dAB = (pB - pA).normalized()
        if dAB.magnitude < 0.5: dAB = Vector((1.0, 0.0))
        cAB = dAB.orthogonal()
        dAB *= halfsize
        cAB *= halfsize

        p00 = pA - dAB - cAB
        p01 = pA - dAB + cAB
        p10 = pB + dAB - cAB
        p11 = pB + dAB + cAB

        glVertex2f(p00[0], p00[1])
        glVertex2f(p01[0], p01[1])
        glVertex2f(p11[0], p11[1])
        glVertex2f(p10[0], p10[1])

        glEnd()
    else:
        locations_2d = [bpy_extras.view3d_utils.location_3d_to_region_2d(region, rv3d, loc_3d)
                        for loc_3d in self.list_location_3d]
        triangles = tessellate_polygon([locations_2d])

        glBegin(GL_TRIANGLES)
        for tri in triangles:
            for v_id in tri:
                v = locations_2d[v_id]
                glVertex2f(v[0], v[1])
        glEnd()

        if preference().BC_use_mirror:
            if not self.clump_mirror:
                if preference().BoxCutter_mirror_axis_x:
                        draw_mirror(self, context, triangles, locations_2d, "X")
                if preference().BoxCutter_mirror_axis_y:
                        draw_mirror(self, context, triangles, locations_2d, "Y")
                if preference().BoxCutter_mirror_axis_x and preference().BoxCutter_mirror_axis_y:
                    draw_mirror(self, context, triangles, locations_2d, "XY")

        if self.draw_outline:
            glLineWidth(preference().BoxCutter_curveline_width)
            if self.use_3d_cursor:
                bgR, bgG, bgB, bgA = preference().BC_cursorline_color
                glColor4f(bgR, bgG, bgB, bgA)
            else:
                bgR, bgG, bgB, bgA = preference().BC_curveline_color
                glColor4f(bgR, bgG, bgB, bgA)
            glBegin(GL_LINE_LOOP)
            for loc_2d in locations_2d:
                glVertex2f(loc_2d[0], loc_2d[1])
            glEnd()

        if self.draw_points:
            glPointSize(preference().BoxCutter_curvepoints_size)
            bgR, bgG, bgB, bgA = preference().BC_curveline_color
            glColor4f(bgR, bgG, bgB, bgA)
            glBegin(GL_POINTS)
            for curve_point in self.curve_points:
                loc_2d = curve_point["2d"]
                glVertex2f(loc_2d[0], loc_2d[1])
            glEnd()

        if self.draw_axis:
            all_polygon_points = []
            for p in self.curve_points:
                all_polygon_points.append(p["2d"])

            i = self.drag_point_id
            if i < 0:
                center_point = centroid(all_polygon_points)
            else:
                for p in self.curve_points:
                    center_point = p["2d"]
            glColor4f(0, 0, 0, 1)
            glLineStipple(3, 0xAAAA)
            glLineWidth(1.0)
            glEnable(GL_LINE_STIPPLE)
            glBegin(GL_LINES)
            height, width = region_dimensions()
            if self.use_x_axis:
                glVertex2f(center_point[0] - width, center_point[1])
                glVertex2f(center_point[0] + width, center_point[1])
            else:
                glVertex2f(center_point[0], center_point[1] + height)
                glVertex2f(center_point[0], center_point[1] - height)
            glEnd()
            glDisable(GL_LINE_STIPPLE)

        if self.drag_mode in {'SCALE', 'ROTATE'}:
            glColor4f(0, 0, 0, 1)
            glLineStipple(3, 0xAAAA)
            glLineWidth(1.0)
            glEnable(GL_LINE_STIPPLE)
            glBegin(GL_LINES)
            glVertex2f(self.start_center.x, self.start_center.y)
            glVertex2f(self.drag_start.x, self.drag_start.y)
            glEnd()
            glDisable(GL_LINE_STIPPLE)

        if self.drag_mode is 'ROTATE':
            if self.cal_angle not in {0, -360}:
                all_polygon_points = []
                for p in self.curve_points:
                    all_polygon_points.append(p["2d"])
                center_point = centroid(all_polygon_points)
                draw_button_text(True, center_point[0]-8, center_point[1], '{}'.format(-(abs(180-(-self.cal_angle))-180)), preference().BoxCutter_display_text_size)

        if self.drag_mode in {'WEIGHT', None, 'ROTATE', 'SCALE'}:
            glPointSize(preference().BoxCutter_verts_size)
            bgR, bgG, bgB, bgA = preference().BC_verts_color
            glColor4f(bgR, bgG, bgB, bgA)
            glBegin(GL_POINTS)
            for loc_2d in locations_2d:
                glVertex2f(loc_2d[0], loc_2d[1])
            glEnd()

        if self.drag_mode in {'WEIGHT', None}:
            all_polygon_points = []
            for p in self.curve_points:
                all_polygon_points.append(p["2d"])
            center_point = centroid(all_polygon_points)
            draw_button_text(True, center_point[0], center_point[1], '{}' .format(preference().BoxCutter_curve_vert_count), preference().BoxCutter_display_text_size)

    glLineWidth(1)
    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)


def draw_mirror(self, context, triangles, locations_2d, axis):

    object = bpy.context.active_object
    if object is not None:
        if preference().BoxCutter_mirror_type == "3D":
            origin = point_to_2d(self, context, self.first_active.location[0], self.first_active.location[1], self.first_active.location[2])
        else:
            origin = point_to_2d(self, context, bpy.context.space_data.cursor_location[0], bpy.context.space_data.cursor_location[1], bpy.context.space_data.cursor_location[2])
    else:
        origin = point_to_2d(self, context, 0, 0, 0)

    glBegin(GL_TRIANGLES)
    for tri in triangles:
        for v_id in tri:
            v = locations_2d[v_id]
            mirror_v = [scale(origin, v, axis)]
            glVertex2f(mirror_v[0][0], mirror_v[0][1])
    glEnd()
