import bpy
from bgl import *
import blf
import bpy_extras.view3d_utils
import math

from math import pi, cos, sin
from mathutils import Vector
from .. preferences import BC_border_color, BC_indicator_color1, BC_indicator_color2, BC_grid_color, preference
from .. utils.blender_ui import region_dimensions, view_3d_tools_panel_overlay_width
from .. utils.space_3d import center_point, region3d
from .. utils.blender_ui import dpi_factor, dpi

def draw_boolean_region(self, context):
    dx0 = self.first_mouse_x
    dy0 = self.first_mouse_y
    dx1 = self.mouse_path[0]
    dy1 = self.mouse_path[1]

    self.last_x_position = dx1
    self.last_y_position = dy1

    linex = math.sqrt(math.pow(dx0-dx0, 2) + math.pow(dy1-dy0, 2))
    liney = math.sqrt(math.pow(dx1-dx0, 2) + math.pow(dy0-dy0, 2))

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)

    if linex < liney and dx0 > dx1:
        glColor4f(0.62, 0.5, 0.2, 0.5)
        self.color = 1
    elif linex < liney and dx0 < dx1:
        glColor4f(0.5, 0.5, 0.5, 0.5)
        self.color = 2
    elif linex > liney and dy0 < dy1:
        glColor4f(0.29, 0.52, 1.0, 0.53)
        self.color = 3
    elif linex > liney and dy0 > dy1:
        glColor4f(0.6, 0.2, 0.2, 0.5)
        self.color = 4

    glLineWidth(3)

    glBegin(GL_LINE_STRIP)
    glVertex2f(dx0, dy0)
    glVertex2f(dx0, dy1)
    glVertex2f(dx1, dy1)
    glVertex2f(dx1, dy0)
    glVertex2f(dx0, dy0)

    glEnd()

    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)

def draw_border_region(self, context):
    region = context.region

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)
    glColor4f(BC_border_color()[0], BC_border_color()[1], BC_border_color()[2], BC_border_color()[3])
    # lw
    glLineWidth(preference().BC_border_size)

    glBegin(GL_LINE_LOOP)
    offset = preference().BC_border_offset
    panels_l = view_3d_tools_panel_overlay_width(bpy.context.area, "left")
    panels_r = view_3d_tools_panel_overlay_width(bpy.context.area, "right")

    glVertex2f(0 + offset + panels_l, 0 + offset)
    glVertex2f(0 + offset + panels_l, region.height - offset)
    glVertex2f(region.width - offset - panels_r, region.height - offset)
    glVertex2f(region.width - offset - panels_r, 0 + offset)

    glEnd()
    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)

def draw_extending_slicengon(self, context):
    region = context.region
    rv3d = region3d(self, context)

    self.last_x_position = self.mouse_path[0]
    self.last_y_position = self.mouse_path[1]

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)
    if self.first_active in list(bpy.context.selected_objects):
        glColor4f(0.62, 0.5, 0.2, 0.5)
    else:
        glColor4f(0.5, 0.5, 0.5, 0.5)

    glPointSize(5)
    glLineWidth(3)

    if 1 <= len(self.list_location_3d) < 2:
        glBegin(GL_LINES)
    else:
        glBegin(GL_TRIANGLE_FAN)

    glBegin(GL_TRIANGLE_FAN)
    for x in self.list_location_3d:
        loc_1 = bpy_extras.view3d_utils.location_3d_to_region_2d(region, rv3d, x)
        glVertex2f(loc_1[0], loc_1[1])
    glVertex2f(self.mouse_path[0], self.mouse_path[1])
    glEnd()
    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)

def draw_extending_ngon(self, context):
    region = context.region
    rv3d = region3d(self, context)

    self.last_x_position = self.mouse_path[0]
    self.last_y_position = self.mouse_path[1]

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)

    if self.first_active in list(bpy.context.selected_objects):
        glColor4f(0.6, 0.2, 0.2, 0.5)
    else:
        glColor4f(0.5, 0.5, 0.5, 0.5)

    glPointSize(5)
    glLineWidth(3)

    if 1 <= len(self.list_location_3d) < 2:
        glBegin(GL_LINES)
    else:
        glBegin(GL_TRIANGLE_FAN)

    for x in self.list_location_3d:
        loc_1 = bpy_extras.view3d_utils.location_3d_to_region_2d(region, rv3d, x)
        glVertex2f(loc_1[0], loc_1[1])
    glVertex2f(self.mouse_path[0], self.mouse_path[1])
    glEnd()
    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)

def draw_box(x0, y0, width, height, grid_offset=0):
    x1 = x0 + width
    y1 = y0 - height/2
    y0 += height/2

    bgR = BC_indicator_color1()[0]
    bgG = BC_indicator_color1()[1]
    bgB = BC_indicator_color1()[2]
    bgA = BC_indicator_color1()[3]

    position = [[x0, y0], [x0, y1], [x1, y1], [x1, y0]]

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)

    for mode in [GL_QUADS]:
        glBegin(mode)
        glColor4f(bgR, bgG, bgB, bgA)
        for v1, v2 in position:
            glVertex2f(v1, v2)
        glEnd()

    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)

def draw_button_text(hover, x, y, text, size=10):
    blf.position(0, x, y, 0)
    blf.size(0, size, dpi())
    if hover:
        txtcol = bpy.context.user_preferences.themes[0].user_interface.wcol_pie_menu.text
    else:
        txtcol = bpy.context.user_preferences.themes[0].user_interface.wcol_pie_menu.text_sel
    txR = txtcol[0]
    txG = txtcol[1]
    txB = txtcol[2]
    txA = 1.0
    glColor4f(txR, txG, txB, txA)
    blf.draw(0, text)

def draw_grid(self, context):
    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)
    glColor4f(BC_grid_color()[0], BC_grid_color()[1], BC_grid_color()[2], BC_grid_color()[3])
    region_height, region_width = region_dimensions()
    center = center_point(self, context)
    grid_path_spaceing = preference().BoxCutter_grid_spaceing // context.space_data.region_3d.view_distance

    if grid_path_spaceing > 10:
        glLineWidth(1)
        glBegin(GL_LINES)
        coord = center[0]
        while coord <= region_width:
            glVertex2f(coord, 0)
            glVertex2f(coord, region_height)
            coord += grid_path_spaceing
        coord = center[0] - grid_path_spaceing
        while coord >= 0:
            glVertex2f(coord, 0)
            glVertex2f(coord, region_height)
            coord -= grid_path_spaceing
        coord = center[1]
        while coord <= region_height:
            glVertex2f(0, coord)
            glVertex2f(region_width, coord)
            coord += grid_path_spaceing
        coord = center[1] - grid_path_spaceing
        while coord >= 0:
            glVertex2f(0, coord)
            glVertex2f(region_width, coord)
            coord -= grid_path_spaceing
        glEnd()
        # glLineWidth(1)
    else:
        blf.position(0, (region_width-450)*dpi_factor(), 15*dpi_factor(), 0)
        blf.size(0, 10, dpi())
        txtcol = bpy.context.user_preferences.themes[0].user_interface.wcol_pie_menu.text_sel
        txR = txtcol[0]
        txG = txtcol[1]
        txB = txtcol[2]
        txA = 1.0
        glColor4f(txR, txG, txB, txA)
        blf.draw(0, 'Increase the grid size or zoom in')

    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)

def draw_point(self, context):
    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)
    glColor4f(0.7, 0, 0.87, 0.9)
    region_height, region_width = region_dimensions()
    center = center_point(self, context)
    grid_path_spaceing = preference().BoxCutter_grid_spaceing // context.space_data.region_3d.view_distance

    grid_path_x = int(int(center[0]) - (int(grid_path_spaceing) * (int((center[0] - self.x) / int(grid_path_spaceing)))))
    grid_path_y = int(int(center[1]) - (int(grid_path_spaceing) * (int((center[1] - self.y) / int(grid_path_spaceing)))))

    glLineWidth(2)
    glBegin(GL_LINES)
    glVertex2f(grid_path_x - 4, grid_path_y + 4)
    glVertex2f(grid_path_x + 4, grid_path_y + 4)
    glVertex2f(grid_path_x + 4, grid_path_y + 4)
    glVertex2f(grid_path_x + 4, grid_path_y - 4)
    glVertex2f(grid_path_x + 4, grid_path_y - 4)
    glVertex2f(grid_path_x - 4, grid_path_y - 4)
    glVertex2f(grid_path_x - 4, grid_path_y - 4)
    glVertex2f(grid_path_x - 4, grid_path_y + 4)
    glEnd()
    glLineWidth(1)

    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)

def draw_view_roll(self, context):

    # TODO: create theme options
    size = 75
    line_width = 1

    theme_x = (0.75, 0.2, 0.15)
    theme_y = (0.57, 0.75, 0.37)
    theme_wire = (0.15, 0.15, 0.15)
    theme_view_line = (0.0, 0.0, 0.0)

    glLineWidth(line_width)

    # width_offset = 0.0 if line_width%2 == 0 else 0.5
    width_offset = 0.0

    # x dummy axis
    glColor3f(*theme_wire)
    glEnable(GL_LINE_STIPPLE)
    glLineStipple(3, 0xAAAA)
    glBegin(GL_LINES)
    glVertex2f(self.start_x, self.start_y)
    glVertex2f(self.start_x-size*0.5, self.start_y)
    glEnd()
    glDisable(GL_LINE_STIPPLE)

    # y dummy axis
    glColor3f(*theme_wire)
    glEnable(GL_LINE_STIPPLE)
    glLineStipple(3, 0xAAAA)
    glBegin(GL_LINES)
    glVertex2f(self.start_x, self.start_y)
    glVertex2f(self.start_x, self.start_y-size*0.5)
    glEnd()
    glDisable(GL_LINE_STIPPLE)

    # x axis
    offset_x = (self.start_x + size) - 10
    offset_y = 5

    glColor3f(*theme_x)
    glBegin(GL_LINES)
    glVertex2f(self.start_x, self.start_y)
    glVertex2f(offset_x, self.start_y)
    glEnd()

    # x arrow
    glBegin(GL_TRIANGLES)
    glVertex2f(self.start_x+size, self.start_y+width_offset)
    glVertex2f(offset_x, self.start_y+offset_y+width_offset)
    glVertex2f(offset_x, self.start_y-offset_y+width_offset)
    glEnd()

    # y axis
    offset_y = (self.start_y + size) - 10
    offset_x = 5

    glColor3f(*theme_y)
    glBegin(GL_LINES)
    glVertex2f(self.start_x, self.start_y)
    glVertex2f(self.start_x, offset_y)
    glEnd()

    # y arrow
    glBegin(GL_TRIANGLES)
    glVertex2f(self.start_x+width_offset, self.start_y+size)
    glVertex2f(self.start_x+offset_x+width_offset, offset_y)
    glVertex2f(self.start_x-offset_x+width_offset, offset_y)
    glEnd()

    # roll angle
    glLineWidth(1)

    glColor3f(*theme_view_line)
    glEnable(GL_LINE_STIPPLE)
    glLineStipple(3, 0xAAAA)
    glBegin(GL_LINES)
    glVertex2f(self.start_x, self.start_y)
    glVertex2f(self.mouse_x, self.mouse_y)
    glEnd()
    glDisable(GL_LINE_STIPPLE)

    # angle text
    blf.position(0, self.mouse_x+15, self.mouse_y, 0)
    blf.size(0, 12, dpi())

    txtcol = bpy.context.user_preferences.themes[0].user_interface.wcol_pie_menu.text

    txR = txtcol[0]
    txG = txtcol[1]
    txB = txtcol[2]
    txA = 1.0
    glColor4f(txR, txG, txB, txA)

    if self.numeric_entry:
        if self.numeric_user_input:
            blf.draw(0, "{}".format(self.numeric_user_input))

        else:
            blf.draw(0, "")

    elif not self.use_snap:
        blf.draw(0, "{}{:0.2f}".format("" if self.is_positive else "-", self.roll_angle))

    else:
        blf.draw(0, "{}{}".format("" if self.is_positive else "-", round(self.roll_angle)))

def anchor_line(location, mouse, color=(0.1, 0.1, 0.1)):
    glEnable(GL_BLEND)
    glEnable(GL_LINE_STIPPLE)

    glLineStipple(6, 0xAAAA)

    glColor3f(*color)

    glBegin(GL_LINES)
    glVertex2f(location.x, location.y)
    glVertex2f(mouse.x, mouse.y)
    glEnd()

    glDisable(GL_LINE_STIPPLE)
    glDisable(GL_BLEND)

def text(location, text, size=18, color=(0.8, 0.8, 0.8), frame=True):
    blf.size(0, size, dpi())

    if frame:
        text_frame(location, Vector(blf.dimensions(0, text)))

    glEnable(GL_BLEND)

    blf.position(0, location.x, location.y, 0)

    glColor3f(*color)

    blf.draw(0, text)

    glDisable(GL_BLEND)

def text_frame(location, dimension, padding=10, color=(0.1, 0.1, 0.1, 0.5)):
    #TODO: rounded corners, border
    points = (
        Vector((location.x - padding, location.y - padding)),
        Vector((location.x + dimension.x + padding, location.y - padding)),
        Vector((location.x + dimension.x + padding, location.y + dimension.y + padding)),
        Vector((location.x - padding, location.y + dimension.y + padding)))

    glEnable(GL_BLEND)
    glBegin(GL_QUADS)

    glColor4f(*color)

    for point in points:
        glVertex2f(*point)

    glEnd()
    glDisable(GL_BLEND)

def dot(location, size=8, color=(0.1, 0.1, 0.1, 0.4)):
    glEnable(GL_BLEND)

    glColor4f(*color)

    glBegin(GL_TRIANGLE_FAN)
    glVertex2f(*location)

    # 1/(32 sides) = 0.03125, performance tweak
    points = ((location.x + cos(i * pi * 2 * 0.03125) * size, location.y + sin(i * pi * 2 * 0.03125) * size) for i in range(33))
    for point in points:
        glVertex2f(*point)

    glEnd()

    glDisable(GL_BLEND)
