import bpy
from bgl import *
from .. utils.blender_ui import dpi_factor, view_3d_tools_panel_overlay_width
from . draw_px import draw_border_region
from . logo import draw_logo_boxcutter
from . draw_hud import draw_hud
from .. utils.objects import current_selected_status, set_active
from .. preferences import enable_border_display, preference


def draw(self, context):

    if not (DrawBooleanLayout.running_boxcutters.get(self._region) is self): return
    if self._region != context.region: return
    draw_logo_boxcutter()

    if preference().BC_switch:
        xy_start = [view_3d_tools_panel_overlay_width(bpy.context.area, "left"), bpy.context.region.height - vertical_offset()]
        xy_scale = [dpi_factor(), dpi_factor()]

        draw_hud(self, context, xy_start, xy_scale)

        if enable_border_display():
            draw_border_region(self, context)

# ============ from dairin0d's library ============ #


def point_in_rect(p, r):
    return ((p[0] >= r.x) and (p[0] < r.x + r.width) and (p[1] >= r.y) and (p[1] < r.y + r.height))


def rv3d_from_region(area, region):
    if (area.type != 'VIEW_3D') or (region.type != 'WINDOW'): return None

    space_data = area.spaces.active
    try:
        quadviews = space_data.region_quadviews
    except AttributeError:
        quadviews = None # old API

    if not quadviews: return space_data.region_3d

    x_id = 0
    y_id = 0
    for r in area.regions:
        if (r.type == 'WINDOW') and (r != region):
            if r.x < region.x: x_id = 1
            if r.y < region.y: y_id = 1

    # 0: bottom left (Front Ortho)
    # 1: top left (Top Ortho)
    # 2: bottom right (Right Ortho)
    # 3: top right (User Persp)
    return quadviews[y_id | (x_id << 1)]

# areas can't overlap, but regions can


def ui_contexts_under_coord(x, y, window=None):
    point = int(x), int(y)
    if not window: window = bpy.context.window
    screen = window.screen
    scene = screen.scene
    tool_settings = scene.tool_settings
    for area in screen.areas:
        if point_in_rect(point, area):
            space_data = area.spaces.active
            for region in area.regions:
                if point_in_rect(point, region):
                    yield dict(window=window, screen=screen,
                               area=area, space_data=space_data, region=region,
                               region_data=rv3d_from_region(area, region),
                               scene=scene, tool_settings=tool_settings)
            break


def region_exists(r):
    wm = bpy.context.window_manager
    for window in wm.windows:
        for area in window.screen.areas:
            for region in area.regions:
                if region == r: return True
    return False
# ================================================= #


class DrawBooleanLayout(bpy.types.Operator):
    bl_idname = "boxcutter.draw_boolean_layout"
    bl_label = "Draw Screen Buttons"
    bl_description = "draw screen buttons"

    x = 0
    y = 0

    global show_hops

    running_boxcutters = {}

    def validate_region(self):
        if not (DrawBooleanLayout.running_boxcutters.get(self._region) is self): return False
        return region_exists(self._region)

    def modal(self, context, event):
        active_object, other_objects, other_object = current_selected_status()

        try:
            if not self.validate_region():
                self.cancel(context)
                return {'CANCELLED'}

            context.area.tag_redraw()

            self.LMB = False
            self.RMB = False

            if event.type == 'MOUSEMOVE':
                self.x = event.mouse_region_x
                self.y = event.mouse_region_y
                x_abs = self.x + context.region.x
                y_abs = self.y + context.region.y
                ui_contexts = list(ui_contexts_under_coord(x_abs, y_abs, context.window))
                if ui_contexts and ui_contexts[0]:
                    self._region_mouse = [uic["region"] for uic in ui_contexts]

            if not (self._region in self._region_mouse): return {'PASS_THROUGH'}

            if preference().BC_switch:
                if event.type == 'LEFTMOUSE'and event.value == 'PRESS':

                    # LMB + ctr/shif/alt hotkeys
                    if (active_object is None) or (active_object.type == "MESH" and active_object.mode == "OBJECT"):
                        if preference().BoxCutter_set_cutter_shape == "BOX":
                            if len(bpy.context.selected_objects) > 0:
                                if preference().BoxCutter_enable_borders and len(bpy.context.selected_objects) > 1:
                                    if event.ctrl:
                                        bpy.ops.boxcutter.draw_border('INVOKE_DEFAULT')
                                else:
                                    if event.ctrl:
                                        if event.shift:
                                            if preference().BoxCutter_allow3d_cutter:
                                                bpy.ops.boxcutter.draw_shape('INVOKE_DEFAULT', mode="SLICE")
                                            else:
                                                bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="SLICE")
                                            return {'RUNNING_MODAL'}
                                        else:
                                            if preference().BoxCutter_allow3d_cutter:
                                                bpy.ops.boxcutter.draw_shape('INVOKE_DEFAULT', mode="CUT")
                                            else:
                                                bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="CUT")
                                            return {'RUNNING_MODAL'}
                            elif len(bpy.context.selected_objects) == 0:
                                if event.ctrl:
                                    if event.shift:
                                        bpy.ops.boxcutter.add_cube()
                                        return {'RUNNING_MODAL'}
                                    else:
                                        bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="MAKE")
                                        return {'RUNNING_MODAL'}

                        elif (preference().BoxCutter_set_cutter_shape == "NGON"):
                            round_corners = (preference().BoxCutter_set_cutter_shape == "NGON")
                            if len(bpy.context.selected_objects) > 0:
                                if preference().BoxCutter_enable_borders and len(bpy.context.selected_objects) > 1:
                                    if event.ctrl:
                                        bpy.ops.boxcutter.draw_border('INVOKE_DEFAULT')
                                else:
                                    if event.ctrl:
                                        if event.shift:
                                            bpy.ops.boxcutter.draw_curve('INVOKE_DEFAULT',  mode="SLICE", round_corners=round_corners)
                                            return {'RUNNING_MODAL'}
                                        else:
                                            bpy.ops.boxcutter.draw_curve('INVOKE_DEFAULT',  mode="CUT", round_corners=round_corners)
                                            return {'RUNNING_MODAL'}
                            elif len(bpy.context.selected_objects) == 0:
                                if event.ctrl:
                                    if event.shift:
                                        bpy.ops.boxcutter.add_cube()
                                        return {'RUNNING_MODAL'}
                                    else:
                                        bpy.ops.boxcutter.draw_curve('INVOKE_DEFAULT', mode="MAKE", round_corners=round_corners)
                                        return {'RUNNING_MODAL'}

                        elif preference().BoxCutter_set_cutter_shape == "CIRCLE":
                            if len(bpy.context.selected_objects) > 0:
                                if preference().BoxCutter_enable_borders and len(bpy.context.selected_objects) > 1:
                                    if event.ctrl:
                                        bpy.ops.boxcutter.draw_border('INVOKE_DEFAULT')
                                else:
                                    if event.ctrl:
                                        if event.shift:
                                            bpy.ops.boxcutter.draw_circle('INVOKE_DEFAULT', mode="SLICE")
                                            return {'RUNNING_MODAL'}
                                        else:
                                            bpy.ops.boxcutter.draw_circle('INVOKE_DEFAULT')
                                            return {'RUNNING_MODAL'}

                            if len(bpy.context.selected_objects) == 0:
                                if event.ctrl:
                                    if event.shift:
                                        bpy.ops.boxcutter.add_cube()
                                        return {'RUNNING_MODAL'}
                                    else:
                                        bpy.ops.boxcutter.draw_circle('INVOKE_DEFAULT', mode="MAKE")
                                        return {'RUNNING_MODAL'}

                    elif active_object.type == "MESH" and active_object.mode == "EDIT":
                        if preference().BoxCutter_set_bc_mode == "CUTTER":
                            if preference().BoxCutter_set_cutter_shape == "BOX":
                                if event.ctrl:
                                    if event.shift:
                                        bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="KNIFE", knife_selection=True)
                                        return {'RUNNING_MODAL'}
                                    else:
                                        bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="KNIFE")
                                        return {'RUNNING_MODAL'}
                            elif preference().BoxCutter_set_cutter_shape == "CIRCLE":
                                if event.ctrl:
                                    if event.shift:
                                        bpy.ops.boxcutter.draw_circle('INVOKE_DEFAULT', mode="KNIFE", knife_selection=True)
                                        return {'RUNNING_MODAL'}
                                    else:
                                        bpy.ops.boxcutter.draw_circle('INVOKE_DEFAULT', mode="KNIFE")
                                        return {'RUNNING_MODAL'}
                            elif (preference().BoxCutter_set_cutter_shape == "NGON"):
                                round_corners = (preference().BoxCutter_set_cutter_shape == "NGON")
                                if event.ctrl:
                                    if event.shift:
                                        bpy.ops.boxcutter.draw_curve('INVOKE_DEFAULT',  mode="KNIFE", round_corners=round_corners, knife_selection=True)
                                        return {'RUNNING_MODAL'}
                                    else:
                                        bpy.ops.boxcutter.draw_curve('INVOKE_DEFAULT',  mode="KNIFE", round_corners=round_corners)
                                        return {'RUNNING_MODAL'}

                        if preference().BoxCutter_set_bc_mode == "MESHMAKER":
                            if event.ctrl:
                                bpy.ops.boxcutter.make_edge('INVOKE_DEFAULT')
                                return {'RUNNING_MODAL'}

                # others

                elif event.type == 'MIDDLEMOUSE'and event.value == 'PRESS' and event.alt and event.shift and not event.ctrl:
                    bpy.ops.boxcutter.view_align('INVOKE_DEFAULT')

                elif event.type == 'MIDDLEMOUSE'and event.value == 'PRESS' and event.alt and event.shift and event.ctrl:
                    bpy.ops.boxcutter.view_roll('INVOKE_DEFAULT')

                elif event.value == 'PRESS' and event.type == 'D' and event.ctrl:
                    bpy.ops.boxcutter.bisect()
                    return {'RUNNING_MODAL'}

                elif event.value == 'PRESS' and event.type == 'D' and not event.ctrl and not event.shift and not event.alt:
                    bpy.ops.wm.call_menu_pie(name='boxcutter_pie_menu')
                    # boxcutter_set_shape()
                    # preference().BoxCutter_shape_moving = False
                    return {'RUNNING_MODAL'}

                elif event.shift and event.type == "Z" and event.value == 'PRESS' and not event.ctrl and not event.alt:
                    if preference().BoxCutter_enable_grid is False:
                        preference().BoxCutter_enable_grid = True
                    else:
                        preference().BoxCutter_enable_grid = False
                    return {'RUNNING_MODAL'}

                elif event.type == "WHEELUPMOUSE" and event.shift and not event.ctrl and not event.alt or event.type == 'NUMPAD_PLUS' and event.shift and event.value == 'PRESS':
                    if preference().BoxCutter_grid_spaceing == 1:
                        preference().BoxCutter_grid_spaceing += 4
                    else:
                        preference().BoxCutter_grid_spaceing += 5
                    return {'RUNNING_MODAL'}

                elif event.type == "WHEELDOWNMOUSE" and event.shift and not event.ctrl and not event.alt or event.type == 'NUMPAD_MINUS' and event.shift and event.value == 'PRESS':
                    preference().BoxCutter_grid_spaceing -= 5
                    return {'RUNNING_MODAL'}

            if event.type == 'ESC' and event.value == 'PRESS':
                self.cancel(context)
                return {'CANCELLED'}

            return {'PASS_THROUGH'}
        except Exception as exc:
            print(exc)
            self.cancel(context)
            return {'CANCELLED'}

    def cancel(self, context):
        bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
        if DrawBooleanLayout.running_boxcutters.get(self._region) is self:
            del DrawBooleanLayout.running_boxcutters[self._region]

    def invoke(self, context, event):
        if context.area.type == 'VIEW_3D':
            #DrawBooleanLayout.running_boxcutters = {}
            self._region_mouse = [context.region]
            self._region = context.region
            DrawBooleanLayout.running_boxcutters[self._region] = self
            args = (self, context)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw, args, 'WINDOW', 'POST_PIXEL')
            context.window_manager.modal_handler_add(self)
            preference().BC_switch = True
            return {'RUNNING_MODAL'}

        else:
            return {'CANCELLED'}


def boxcutter_set_shape():

    if preference().BoxCutter_set_cutter_shape == "BOX":
        preference().BoxCutter_set_cutter_shape = "CIRCLE"
    elif preference().BoxCutter_set_cutter_shape == "CIRCLE":
        if preference().BoxCutter_enable_ngon:
            preference().BoxCutter_set_cutter_shape = "NGON"
        else:
            preference().BoxCutter_set_cutter_shape = "BOX"
    elif preference().BoxCutter_set_cutter_shape == "NGON":
        preference().BoxCutter_set_cutter_shape = "BOX"


def vertical_offset():
    if bpy.context.scene.unit_settings.system == "NONE":
        return 0
    else:
        return 20

    return method
