import bpy
from bgl import *
import blf
import os

from .. utils.blender_ui import dpi_factor, dpi
from .. preferences import preference
from . textbox import TextBox


def draw_help():
    font_info = dict(size=11, line=20, scale=1.0)

    def set_font_scale(scale):
        font_info["scale"] = scale
        blf.size(0, int(font_info["size"] * font_info["scale"]), dpi())

    def draw_text(txt, offset=None):
        y_shift = font_info["line"] * font_info["scale"] * dpi_factor()
        if offset is None:
            blf.position(0, xy[0], xy[1], 0)
            blf.draw(0, txt)
            xy[1] -= y_shift
        else:
            blf.position(0, xy[0], xy[1]+offset*y_shift, 0)
            blf.draw(0, txt)

    set_font_scale(1.0)

    bpy.ops.wm.grab_mouse_position('INVOKE_DEFAULT')
    mouse_pos = bpy.types.WindowManager.mouse_pos
    left_click = bpy.types.WindowManager.left_click
    right_click = bpy.types.WindowManager.right_click

    if not hasattr(bpy.types.WindowManager, "show_help"):
        bpy.types.WindowManager.show_help = False

    helpText = ""
    path = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))

    if preference().BoxCutter_help_item == "MAIN":
        if len(bpy.context.selected_objects) == 0:
            path = os.path.normpath(os.path.join(path, "help", "MAIN", "0.txt"))
        else:
            path = os.path.normpath(os.path.join(path, "help", "MAIN", "1.txt"))
    else:
        path = os.path.normpath(os.path.join(path, "help", preference().BoxCutter_help_item+".txt"))

    if os.path.isfile(path):
        helpTextFile = open(path)
        helpText = helpTextFile.read()
        helpTextFile.close()

    region = bpy.context.region
    width = region.width
    height = region.height
    boxWidth = 300
    boxHeight = 200
    dist = preference().BC_help_button_border_distance
    corner = preference().BC_help_button_corner
    if corner == 0:
        corner = [0, 1]
    if corner == 1:
        corner = [1, 1]
    if corner == 2:
        corner = [0, 0]
    if corner == 3:
        corner = [1, 0]

    mouse_button = right_click

    if bpy.context.user_preferences.inputs.select_mouse == "LEFT":
        mouse_button = left_click

    if helpText != "":

        help_button = TextBox(width-(boxWidth*dist), boxHeight*dist, boxWidth, boxHeight, 10, dist, helpText)

        help_button.collapse()

        if bpy.types.WindowManager.show_help:
            help_button.uncollapse()

        help_button.snap_to_corner(bpy.context, corner=corner)

        over = help_button.hover(mouse_pos[0], mouse_pos[1])

        if over and mouse_button:
            bpy.types.WindowManager.show_help = not bpy.types.WindowManager.show_help

        help_button.draw()


def draw_box(x0, y0, width, height):

    x1 = x0 + width
    y1 = y0 - height/2
    y0 += height/2

    position = [[x0, y0], [x0, y1], [x1, y1], [x1, y0]]

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)

    for mode in [GL_QUADS]:
        glBegin(mode)
        glColor4f(0.1, 0.1, 0.1, 0.65)
        for v1, v2 in position:
            glVertex2f(v1, v2)
        glEnd()

    txtcol = bpy.context.user_preferences.themes[0].user_interface.wcol_pie_menu.text

    txR = txtcol[0]
    txG = txtcol[1]
    txB = txtcol[2]
    txA = 1.0
    glColor4f(txR, txG, txB, txA)
    glColor4f(txR, txG, txB, txA)
    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)
