import bpy

from bpy.types import PropertyGroup
from bpy.props import EnumProperty, StringProperty, BoolProperty

from . preferences import preference

#############################
# Material Option Props
#############################


class BoxCutterMaterialOptions(PropertyGroup):

    material_mode = EnumProperty(
        name="Mode",
        description="",
        items=[
            ("ALL", "All", "Look for all materials."),
            ("OBJECT", "Object", "Look for materials on the active object.")],
        default="ALL")

    active_material = StringProperty(
        name="Select Material",
        description="Assign the secondary material for bool operations.",
        default="")

    force = BoolProperty(
        name="Force",
        description="Force this active material as the material for non-active objects in a bool operation.",
        default=True)

#############################
# Assign Material
#############################


def assign_material(context, object, replace=False, split=False):
    option = context.window_manager.BoxCutter_material_options if not preference().BoxCutter_enable_hops else context.window_manager.Hard_Ops_material_options
    if option.active_material:
        if object:
            if replace:
                old_active = context.active_object
                context.scene.objects.active = object
                while object.material_slots:
                    bpy.ops.object.material_slot_remove()
                bpy.ops.object.material_slot_add()
                object.material_slots[0].material = bpy.data.materials[option.active_material]
                context.scene.objects.active = old_active
            else:
                prepare(context, option, object)
                if not split:
                    add_material(context, option, context.active_object)


def prepare(context, option, object):
    old_active = context.active_object
    context.scene.objects.active = object
    if option.force:
        while object.material_slots:
            bpy.ops.object.material_slot_remove()
        bpy.ops.object.material_slot_add()
        object.material_slots[0].material = bpy.data.materials[option.active_material]
    else:
        bpy.ops.object.material_slot_add()
        object.material_slots[-1].material = bpy.data.materials[option.active_material]
    context.scene.objects.active = old_active


def add_material(context, option, object):
    old_active = context.active_object
    context.scene.objects.active = object
    if len(object.material_slots) > 1:
        if object.material_slots[-1].material != bpy.data.materials[option.active_material]:
            bpy.ops.object.material_slot_add()
            object.material_slots[-1].material = bpy.data.materials[option.active_material]
    else:
        bpy.ops.object.material_slot_add()
        object.material_slots[-1].material = bpy.data.materials[option.active_material]
    context.scene.objects.active = old_active
