import bpy
import bpy_extras.view3d_utils
from mathutils import Vector
from bpy.props import *
from ... graphic.draw_px import draw_boolean_region
from . draw_utils import set_evet_region
from ... preferences import preference, BoxCutter_enable_hops


class BoxCutterDrawBorder(bpy.types.Operator):
    bl_idname = "boxcutter.draw_border"
    bl_label = "Box Cutter Draw Border Selection"
    bl_options = {'REGISTER', 'UNDO'}

    first_mouse_x = 0
    first_mouse_y = 0
    mouse_path = Vector((0, 0))

    remove_cutters = True

    def cancel(self, context):
        bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')

    def execute(self, context):
        selected = context.selected_objects
        object = context.active_object
        cutters = [obj for obj in selected if obj != object]

        apply_cutters = self.remove_cutters

        if self.color == 1:
            bpy.ops.boxcutter.slice_check(remove_cutters=self.remove_cutters)
        elif self.color == 2:
            if object:
                obj.select = False
            if BoxCutter_enable_hops():
                bpy.ops.hops.complex_sharpen()
        elif self.color == 3:
            bpy.ops.boxcutter.bool_union(remove_cutters=self.remove_cutters, apply_cutters=apply_cutters)
            object.select = True
            if BoxCutter_enable_hops():
                if object:
                    bpy.ops.hops.soft_sharpen()
        elif self.color == 4:
            bpy.ops.boxcutter.bool_difference(remove_cutters=self.remove_cutters, apply_cutters=apply_cutters)
            object.select = True
            if BoxCutter_enable_hops():
                if object:
                    bpy.ops.hops.soft_sharpen()

    def modal(self, context, event):
        context.area.tag_redraw()
        if event.type == 'MOUSEMOVE':
            self.mouse_path = Vector((event.mouse_region_x, event.mouse_region_y))

        elif event.type == 'LEFTMOUSE' and event.value == 'RELEASE' and not event.alt:
            self.execute(context)
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')

            return {'FINISHED'}

        elif event.type == 'LEFTMOUSE' and event.value == 'RELEASE' and event.alt:
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')

            return {'FINISHED'}

        elif event.type == 'RIGHTMOUSE':
            self.cancel(context)
            return {'CANCELLED'}

        elif event.type in {'ESC'}:
            self.cancel(context)
            return {'CANCELLED'}

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        self.color = 4
        self.use_3d_cursor = True
        if context.area.type == 'VIEW_3D':

            args = (self, context)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_boolean_region, args, 'WINDOW', 'POST_PIXEL')

            self.mouse_path = Vector((event.mouse_region_x, event.mouse_region_y))

            set_evet_region(self, event, True)
            context.window_manager.modal_handler_add(self)
            return {'RUNNING_MODAL'}
        else:
            self.report({'WARNING'}, "View3D not found, cannot run operator")
            return {'CANCELLED'}
