import bpy
import bpy_extras.view3d_utils
from mathutils import Vector
from bpy.props import *
from ... utils.modifiers import live
from ... utils.objects import set_active, show_obj_wires
from ... graphic.draw_operators.box import draw_extending_box
from ... utils.space_3d import point_to_2d, context_cursor, region3d, region_exists, ui_contexts_under_coord
from . draw_utils import set_evet_region, check_if_moving, boxcutter_show_wire, generate_3d_mesh, saved_cords, set_grid_path, generate_panel_mesh, project_knife_mesh, cursor3d_follow
from ... preferences import preference, BoxCutter_enable_hops
from ... utils.view_3d import scale, set_viewport_ortho
from .. boolean_operators.split_check import slash
from .. boolean_operators.bool_bmesh import bc_diff_boolean


def draw_box(self, context):
    if not (BoxCutterDrawPoly.running_boxcutters.get(self._region) is self): return
    if self._region != context.region: return
    draw_extending_box(self, context)

saved_points = []

modes = [
    ("CUT", "Cut", ""),
    ("SLICE", "Slice", ""),
    ("MAKE", "Make", ""),
    ("KNIFE", "Knife", ""),
    ("PANEL", "Panel", "")]


class BoxCutterDrawPoly(bpy.types.Operator):
    bl_idname = "boxcutter.draw_poly"
    bl_label = "Box Cutter Draw Poly"
    bl_options = {'REGISTER', 'UNDO'}

    mode = EnumProperty(name="", default="CUT", options={"SKIP_SAVE"}, items=modes)

    run_curve = BoolProperty(name="Run Curve",
                             description="run curve",
                             default=False)

    clump_mirror = BoolProperty(name="Clump Mirror",
                                description="Clump Mirror",
                                default=False)

    knife_selection = BoolProperty(name="KNIFE On Selection",
                                   description="KNIFE On Selection",
                                   default=False)

    cut_through = BoolProperty(name="KNIFE cut through",
                               description="KNIFE cut through",
                               default=True)

    editmode = False
    remove_cutters = True
    apply_cutters = True

    position_2d = []
    position_2d_mx = []
    position_2d_my = []
    cursor3d_pos = 0
    first_mouse_x = 0
    first_mouse_y = 0
    mouse_path = Vector((0, 0))
    savedx = 0
    savedy = 0

    running_boxcutters = {}

    def validate_region(self):
        if not (BoxCutterDrawPoly.running_boxcutters.get(self._region) is self): return False
        return region_exists(self._region)

    def draw(self, context):
        layout = self.layout
        box = layout.box()
        col = box.column()
        col.label("Box Cutter")

    def __del__(self):
        try:
            if self.run_curve:
                for v in self.position_2d:
                    saved_points.append(v)

                bpy.ops.boxcutter.draw_curve('INVOKE_DEFAULT', mode=self.mode, round_corners=True, use_saved_points=True, prevent_move=True, knife_selection=self.knife_selection, enable_adding_points=False, start_shape="BOX")
        except Exception:
            pass

    def execute(self, context):
        if context.active_object is not None:
            if context.active_object.mode == 'EDIT':
                bpy.ops.mesh.select_all(action='SELECT')
                bpy.ops.object.mode_set(mode='OBJECT')
                self.remove_cutters = True
                self.apply_cutters = True
            elif context.active_object.mode == 'OBJECT':
                if preference().workflow == "NONDESTRUCTIVE":
                    self.remove_cutters = False
                    self.apply_cutters = False

        preference().BoxCutter_rotation_value = 0
        boxcutter_show_wire(self, context, False, self.wire)
        self.update_points(context)

        generate_3d_mesh(self, context, 'CIRCLE')

        if self.mode != "MAKE":
            if self.mode == "SLICE":
                bpy.context.scene.objects.active = self.first_active
                if self.editmode:
                    slash(self, context, self.remove_cutters, True, self.first_selected_objects)
                else:
                    slash(self, context, self.remove_cutters, False, self.first_selected_objects)
            elif self.mode == "CUT":
                for obj in self.first_selected_objects:
                    obj.select = True
                if self.editmode:
                    bc_diff_boolean(context, True, self.apply_cutters, self.remove_cutters)
                else:
                    bc_diff_boolean(context, False, self.apply_cutters, self.remove_cutters)
                set_active(self.first_active, True)
            elif self.mode == "PANEL":
                bpy.context.scene.objects.active = self.first_active
                generate_panel_mesh(self, context)
            elif self.mode == "KNIFE":
                bpy.context.scene.objects.active = self.first_active
                project_knife_mesh(self, context, self.cut_through)
            if BoxCutter_enable_hops():
                if self.mode == "CUT":
                    if not self.editmode:
                        for obj in self.first_selected_objects:
                            obj.select = True
                            if obj.hops.status in ("CSHARP", "CSTEP"):
                                bpy.ops.hops.soft_sharpen()
        else:
            if not preference().grey_box_select:
                context.active_object.select = False
            if self.editmode:
                bpy.context.scene.objects.active = self.first_active
                self.first_active.select = True
                bpy.ops.object.join()
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.select_all(action='INVERT')

        preference().BoxCutter_shape_moving = False
        preference().BoxCutter_help_item = 'MAIN'

        return {'FINISHED'}

    def cancel(self, context):
        preference().BoxCutter_rotation_value = 0
        bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
        self.list_location_3d[:] = []
        self.list_location_3d_mx[:] = []
        self.list_location_3d_my[:] = []
        self.list_location_3d_mxy[:] = []
        preference().BoxCutter_shape_moving = False
        preference().BoxCutter_help_item = 'MAIN'
        boxcutter_show_wire(self, context, False, self.wire)
        if BoxCutterDrawPoly.running_boxcutters.get(self._region) is self:
            del BoxCutterDrawPoly.running_boxcutters[self._region]

    def modal(self, context, event):

        context.area.tag_redraw()

        if event.type == 'MOUSEMOVE':
            cursor3d_follow(self, context)
            grid_path_x, grid_path_y = set_grid_path(self, context, event)
            self.mouse_path = Vector((grid_path_x, grid_path_y))

        elif event.type == 'LEFTMOUSE':
            if event.value == 'RELEASE':
                if event.alt:
                    if event.shift:
                        self.remove_cutters = False
                        self.apply_cutters = False
                    self.cut_through = False
                    check_if_moving(self, context, event)
                    set_evet_region(self, event, False)
                    bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                    self.execute(context)
                else:
                    if event.shift:
                        self.remove_cutters = False
                        self.apply_cutters = False
                    check_if_moving(self, context, event)
                    set_evet_region(self, event, False)
                    bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                    self.use_3d_cursor = False
                    self.execute(context)
            return {'FINISHED'}

        elif event.type == 'ONE' and event.value == 'PRESS':
            preference().BoxCutter_mirror_axis_x = not preference().BoxCutter_mirror_axis_x

        elif event.type == 'TWO' and event.value == 'PRESS':
            preference().BoxCutter_mirror_axis_y = not preference().BoxCutter_mirror_axis_y

        elif event.type == 'M' and event.value == 'PRESS':
            preference().BC_use_mirror = not preference().BC_use_mirror

        elif event.type == 'W' and event.value == 'PRESS':
            self.wire = not self.wire
            show_obj_wires(context, self.wire)

        elif event.type == 'RIGHTMOUSE':
            self.cancel(context)
            return {'CANCELLED'}

        elif event.type in {'ESC'}:
            self.cancel(context)
            return {'CANCELLED'}

        elif event.type == 'TAB' and event.value == 'PRESS':
            if self.mode in {"CUT", "SLICE", "PANEL", "MAKE", "KNIFE"}:
                if preference().BoxCutter_shape_moving:
                    self.first_mouse_x = self.mouse_path[0]+self.savedx
                    self.first_mouse_y = self.mouse_path[1]+self.savedy
                self.cancel(context)
                self.run_curve = True
            return {'CANCELLED'}

        elif event.type == 'R' and event.value == 'PRESS':
            if preference().BoxCutter_shape_moving is False:
                self.savedx = -(self.mouse_path[0] - self.first_mouse_x)
                self.savedy = -(self.mouse_path[1] - self.first_mouse_y)
                preference().BoxCutter_shape_moving = True
            if not event.shift:
                self.angle -= preference().BoxCutter_rotation_angle
                preference().BoxCutter_rotation_value = self.angle
            elif event.shift:
                self.angle += preference().BoxCutter_rotation_angle
                preference().BoxCutter_rotation_value = self.angle

        elif event.type == 'SPACE' and event.value == 'PRESS' and not event.shift:
            self.savedx = -(self.mouse_path[0] - self.first_mouse_x)
            self.savedy = -(self.mouse_path[1] - self.first_mouse_y)

            preference().BoxCutter_shape_moving = not preference().BoxCutter_shape_moving

        elif event.type == 'V' and event.value == 'PRESS' and not event.alt and not event.shift:
            self.savedx = saved_cords[0]
            self.savedy = saved_cords[1]
            preference().BoxCutter_shape_moving = True

        elif event.type == 'C' and event.value == 'PRESS' and not event.alt and not event.shift:
            self.savedx = -(self.mouse_path[0] - self.first_mouse_x)
            self.savedy = -(self.mouse_path[1] - self.first_mouse_y)

            saved_cords[0] = self.savedx
            saved_cords[1] = self.savedy

            preference().BoxCutter_shape_moving = not preference().BoxCutter_shape_moving

        elif event.type == 'X':
            if event.value == 'PRESS':
                if self.editmode:
                    if event.value == 'PRESS':
                        if self.mode != "SLICE":
                            self.mode = "SLICE"
                        else:
                            self.mode = "KNIFE"
                else:
                    if self.mode != "SLICE":
                        self.mode = "SLICE"
                    else:
                        self.mode = "CUT"
        elif event.type == 'A':
            if event.value == 'PRESS':
                if self.editmode:
                    if event.value == 'PRESS':
                        if self.mode != "MAKE":
                            self.mode = "MAKE"
                        else:
                            self.mode = "KNIFE"
                else:
                    if self.mode != "MAKE":
                        self.mode = "MAKE"
                    else:
                        self.mode = "CUT"
        elif event.type == 'Z':
            if event.value == 'PRESS':
                if event.shift:
                    preference().BoxCutter_enable_grid = not preference().BoxCutter_enable_grid
                else:
                    if self.editmode:
                        if event.value == 'PRESS':
                            if self.mode != "CUT":
                                self.mode = "CUT"
                            else:
                                self.mode = "KNIFE"
                    else:
                        if self.mode != "PANEL":
                            self.mode = "PANEL"
                        else:
                            self.mode = "CUT"

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):

        saved_points[:] = []
        self.running_boxcutters = []
        preference().BoxCutter_help_item = 'BOX'

        removeable = []
        for live_name in live:
            shape = False
            for object in bpy.data.objects:
                if live_name in bpy.data.objects and object == bpy.data.objects[live_name]:
                    shape = True
            if not shape:
                removeable.append(live_name)
        for live_name in removeable:
            live.remove(live_name)

        self.angle = 0
        self.use_3d_cursor = True
        if context.area.type == 'VIEW_3D':
            if preference().BoxCutter_enter_ortho:
                set_viewport_ortho(context)
            vx, vy, vz = context_cursor(context)
            self.cursor3d_pos = Vector((vx, vy, vz))
            cursor3d_follow(self, context)
            self.first_active = bpy.context.scene.objects.active
            self.first_selected_objects = bpy.context.selected_objects
            if context.active_object is not None:
                if context.active_object.mode == 'EDIT':
                    self.editmode = True
                self.wire = bpy.context.object.show_wire
            else:
                self.wire = None
            self._region_mouse = [context.region]
            self._region = context.region
            BoxCutterDrawPoly.running_boxcutters[self._region] = self
            boxcutter_show_wire(self, context, True, self.wire)
            args = (self, context)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_box, args, 'WINDOW', 'POST_PIXEL')

            grid_path_x, grid_path_y = set_grid_path(self, context, event)
            self.mouse_path = Vector((grid_path_x, grid_path_y))

            self.list_location_3d = []
            self.list_location_3d_mx = []
            self.list_location_3d_my = []
            self.list_location_3d_mxy = []

            self.x = event.mouse_region_x
            self.y = event.mouse_region_y
            x_abs = self.x + context.region.x
            y_abs = self.y + context.region.y

            ui_contexts = list(ui_contexts_under_coord(x_abs, y_abs, context.window))
            if ui_contexts and ui_contexts[0]:
                self._region_mouse = [uic["region"] for uic in ui_contexts]

            set_evet_region(self, event, True)
            context.window_manager.modal_handler_add(self)
            return {'RUNNING_MODAL'}
        else:
            self.report({'WARNING'}, "View3D not found, cannot run operator")
            return {'CANCELLED'}

    def update_points(self, context):

        object = bpy.context.active_object

        region = context.region
        rv3d = region3d(self, context)

        if object is not None:

            if preference().BoxCutter_mirror_type == "3D":
                    origin = point_to_2d(self, context, self.first_active.location[0], self.first_active.location[1], self.first_active.location[2])
            else:
                origin = point_to_2d(self, context, bpy.context.space_data.cursor_location[0], bpy.context.space_data.cursor_location[1], bpy.context.space_data.cursor_location[2])
        else:
            origin = point_to_2d(self, context, 0, 0, 0)

        if self.mode in {"CUT", "SLICE", "PANEL"}:
            local_bbox_center = 0.125 * sum((Vector(b) for b in object.bound_box), Vector())
            global_bbox_center = object.matrix_world * local_bbox_center

            if self.use_3d_cursor:
                depth_location = context_cursor(context)
            else:
                depth_location = global_bbox_center
        else:
            depth_location = context_cursor(context)

        for v in self.position_2d:
            v_3d = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, v, depth_location)
            self.list_location_3d.append(v_3d)

            mirror_x = scale(origin, v, "X")
            mirror_y = scale(origin, v, "Y")
            mirror_xy = scale(origin, v, "XY")

            location_3d_x = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, mirror_x, depth_location)
            location_3d_y = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, mirror_y, depth_location)
            location_3d_xy = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, mirror_xy, depth_location)

            self.list_location_3d_mx.append(location_3d_x)
            self.list_location_3d_my.append(location_3d_y)
            self.list_location_3d_mxy.append(location_3d_xy)
