import math
import time
import bpy
import bpy_extras.view3d_utils
from mathutils import Vector
from mathutils.geometry import interpolate_bezier
from bpy.props import *
from ... utils.modifiers import live
from ... utils.objects import set_active, show_obj_wires
from ... graphic.draw_operators.curve import draw_extending_curve
from . draw_utils import set_evet_region, boxcutter_show_wire, generate_3d_mesh, set_grid_path, generate_panel_mesh, project_knife_mesh, cursor3d_follow
from ... preferences import preference, BoxCutter_enable_hops
from . draw_circle_cutter import saved_circle_points
from . draw_box_cutter import saved_points
from ... utils.space_3d import context_cursor, point_to_2d, region3d, region_exists, ui_contexts_under_coord
from ... utils.view_3d import rotate, centroid, calculate_angle, scale, set_viewport_ortho
from .. boolean_operators.split_check import slash
from .. boolean_operators.bool_bmesh import bc_diff_boolean


#TODO: add to a utils script of some sort
def cross(a, b):
    c = [a[1] * b[2] - a[2] * b[1],
         a[2] * b[0] - a[0] * b[2],
         a[0] * b[1] - a[1] * b[0]]

    return c[2]


def draw_curve(self, context):
    if not (BoxCutterDrawCurve.running_boxcutters.get(self._region) is self): return
    if self._region != context.region: return
    draw_extending_curve(self, context)

modes = [
    ("CUT", "Cut", ""),
    ("SLICE", "Slice", ""),
    ("MAKE", "Make", ""),
    ("KNIFE", "Knife", ""),
    ("PANEL", "Panel", "")]

start_shapes = [
    ("BOX", "Box", ""),
    ("CIRCLE", "Circle", "")]


class BoxCutterDrawCurve(bpy.types.Operator):
    bl_idname = "boxcutter.draw_curve"
    bl_label = "Box Cutter Draw Ngon/Curve"
    bl_options = {'REGISTER', 'UNDO'}

    mode = EnumProperty(name="", default="CUT", options={"SKIP_SAVE"}, items=modes)

    start_shape = EnumProperty(name="", default="BOX", options={"SKIP_SAVE"}, items=start_shapes)

    round_corners = BoolProperty(name="Round corners",
                                 description="Round corners",
                                 default=False)

    resolution = IntProperty(name="Resolution",
                             description="Curve segment resolution",
                             default=12, min=1)

    point_radius = FloatProperty(name="Point radius",
                                 description="Point radius",
                                 default=6.0, min=2.0)

    merge_radius = FloatProperty(name="Merge radius",
                                 description="Coinciding vertices removal radius",
                                 default=1.0, min=0.0)

    snap_angle = BoolProperty(name="Snap angle",
                              description="Snap angle",
                              default=False)

    angle_subdivs = IntProperty(name="Angle subdivs",
                                description="Angle subdivisions",
                                default=72, min=2)

    use_bevel_offset = BoolProperty(name="Use bevel offset",
                                    description="Use bevel offset",
                                    default=True)

    use_saved_points = BoolProperty(name="Use saved points",
                                    description="Use Saved Points",
                                    default=False)

    clump_mirror = BoolProperty(name="Clump Mirror",
                                description="Clump Mirror",
                                default=False)

    prevent_move = BoolProperty(name="Prevent Move",
                                description="Prevent Move",
                                default=False)

    rotation_value = IntProperty(name="Rotation Angle",
                                 description="Rotation Angle",
                                 default=0)

    scale_mode = BoolProperty(name="Scale Mode",
                              description="Scale Mode",
                              default=False)

    use_x_axis = BoolProperty(name="Snap to x or y Axis",
                              description="Snap to x or y Axis",
                              default=True)

    snap_to_axis = BoolProperty(name="Snap To Axis",
                                description="Enable Snap Axis For Move Mode",
                                default=False)

    knife_selection = BoolProperty(name="KNIFE On Selection",
                                   description="KNIFE On Selection",
                                   default=False)

    cut_through = BoolProperty(name="KNIFE cut through",
                               description="KNIFE cut through",
                               default=False)

    enable_adding_points = BoolProperty(name="Enable Adding Points",
                                        description="Enable Adding Points",
                                        default=True)
    editmode = False
    remove_cutters = True
    apply_cutters = True

    lmb_click_time = 0.0
    mmb_click_time = 0.0

    list_location_3d = [1, 1]
    list_location_3d_mx = [1, 1]
    list_location_3d_my = [1, 1]
    list_location_3d_mxy = [1, 1]

    running_boxcutters = {}

    def validate_region(self):
        if not (BoxCutterDrawCurve.running_boxcutters.get(self._region) is self): return False
        return region_exists(self._region)

    def draw(self, context):
        layout = self.layout
        box = layout.box()
        col = box.column()
        col.label("Box Cutter")

    def execute(self, context):
        if context.active_object is not None:
            if context.active_object.mode == 'EDIT':
                bpy.ops.mesh.select_all(action='SELECT')
                bpy.ops.object.mode_set(mode='OBJECT')
                self.remove_cutters = True
                self.apply_cutters = True
            elif context.active_object.mode == 'OBJECT':
                if preference().workflow == "NONDESTRUCTIVE":
                    self.remove_cutters = False
                    self.apply_cutters = False

        boxcutter_show_wire(self, context, False, self.wire)
        self.update_mirror_point_list(context)
        generate_3d_mesh(self, context, 'NGON')

        if self.mode != "MAKE":
            if self.mode == "SLICE":
                bpy.context.scene.objects.active = self.first_active
                if self.editmode:
                    slash(self, context, self.remove_cutters, True, self.first_selected_objects)
                else:
                    slash(self, context, self.remove_cutters, False, self.first_selected_objects)
            elif self.mode == "CUT":
                for obj in self.first_selected_objects:
                    obj.select = True
                if self.editmode:
                    bc_diff_boolean(context, True, self.apply_cutters, self.remove_cutters)
                else:
                    bc_diff_boolean(context, False, self.apply_cutters, self.remove_cutters)
                set_active(self.first_active, True)
            elif self.mode == "PANEL":
                bpy.context.scene.objects.active = self.first_active
                generate_panel_mesh(self, context)
            elif self.mode == "KNIFE":
                bpy.context.scene.objects.active = self.first_active
                project_knife_mesh(self, context, self.cut_through)
            if BoxCutter_enable_hops():
                if self.mode == "CUT":
                    if not self.editmode:
                        for obj in self.first_selected_objects:
                            obj.select = True
                            if obj.hops.status in ("CSHARP", "CSTEP"):
                                bpy.ops.hops.soft_sharpen()
        else:
            if not preference().grey_box_select:
                context.active_object.select = False
            if self.editmode:
                bpy.context.scene.objects.active = self.first_active
                self.first_active.select = True
                bpy.ops.object.join()
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.select_all(action='INVERT')

        return {'FINISHED'}

    def modal(self, context, event):
        context.area.tag_redraw()

        double_click_time = context.user_preferences.inputs.mouse_double_click_time / 1000.0
        cursor3d_follow(self, context)
        grid_path_x, grid_path_y = set_grid_path(self, context, event)
        mouse_pos = Vector((grid_path_x, grid_path_y))
        mouse_pos_exact = Vector((event.mouse_region_x, event.mouse_region_y))

        is_finished = (event.type in {'RET', 'NUMPAD_ENTER'})
        is_cancelled = (event.type in {'ESC', 'RIGHTMOUSE'})

        self.draw_points = (self.drag_mode is None) and (not event.ctrl)
        self.draw_outline = (self.drag_mode is None) and (not event.ctrl)

        self.draw_snap_angle = self.snap_angle and (len(self.curve_points) > 1)
        self.draw_snap_angle &= (self.drag_mode is None) and event.ctrl

        if event.type == 'TAB' and event.value == 'RELEASE':
            self.drag_mode = None
            self.prevent_move = False

        if self.drag_mode is None:
            if event.ctrl:
                if self.enable_adding_points:
                    self.add_cursor_point_to_list(context, False)
                    self.set_point_2d(context, mouse_pos, len(self.curve_points)-1)
            else:
                self.curve_points = self.curve_points[:self.finalized_points]
                self.update_point_list(context)

        if event.type == 'MOUSEMOVE':
            grid_path_x, grid_path_y = set_grid_path(self, context, event)
            self.mouse_path = Vector((grid_path_x, grid_path_y))
            if self.drag_mode == 'MOVE':
                if self.prevent_move:
                        self.drag_mode = None
                        self.prevent_move = False
                else:
                    if event.shift:
                        self.draw_axis = True
                        if self.snap_to_axis:
                            if self.use_x_axis is True:
                                self.move_point_onaxis(context, mouse_pos.x - self.drag_start.x, 'x')
                            else:
                                self.move_point_onaxis(context, mouse_pos.y - self.drag_start.y, 'y')
                        else:
                            if self.is_on_side(context, mouse_pos_exact) in {'LEFT', "RIGHT"}:
                                self.use_x_axis = True
                            elif self.is_on_side(context, mouse_pos_exact) in {'TOP', "BOTTOM"}:
                                self.use_x_axis = False
                            self.snap_to_axis = True
                    else:
                        self.draw_axis = False
                        self.snap_to_axis = False
                        self.move_point(context, mouse_pos - self.drag_start)
                    self.drag_start = mouse_pos
            elif self.drag_mode == 'SCALE':
                oldrange = (self.drag_start.x - self.start_center.x)
                newrange = (1 - 0.0001)
                newvalue = (((mouse_pos_exact.x - self.start_center.x) * newrange) / oldrange) + 0
                self.scale_points(context, newvalue)
                self.drag_start = mouse_pos_exact
            elif self.drag_mode == 'WEIGHT':
                if self.round_corners:
                    delta_weight = (mouse_pos_exact - self.drag_start).x
                    step = preference().BoxCutter_bevel_offset_step
                    if (step > 0) and event.ctrl: delta_weight = round(delta_weight / step) * step
                else:
                    delta_weight = ((mouse_pos_exact - self.drag_start).x / 80.0)
                    if event.ctrl: delta_weight = round(delta_weight / 0.5) * 0.5
                self.add_point_weight(context, delta_weight)
                self.drag_start = mouse_pos_exact
            elif self.drag_mode == 'ROTATE':
                self.rotate_point(context, math.radians(self.cal_angle))

                a = [self.start_center.x, self.start_center.y]
                b = [self.drag_static.x, self.drag_static.y]
                c = [mouse_pos_exact.x, mouse_pos_exact.y]

                # create vectors
                ba = [a[i] - b[i] for i in range(2)]
                ac = [a[i] - c[i] for i in range(2)]

                # rotate
                base = preference().BoxCutter_rotation_angle
                cal_angle = int(base * round(float((360 - calculate_angle(ba, ac))/base)))
                if event.shift: cal_angle = int(round(float((360 - calculate_angle(ba, ac)))))
                self.rotate_point(context, math.radians(cal_angle))
                self.drag_start = mouse_pos_exact
                self.cal_angle = - cal_angle

        elif event.type == 'LEFTMOUSE':
            if event.value == 'PRESS':
                if (time.clock() - self.lmb_click_time) <= double_click_time:
                    if event.alt:
                        self.cut_through = True
                        self.use_3d_cursor = True
                        self.update_point_list(context)
                    if event.shift:
                        self.remove_cutters = False
                        self.apply_cutters = False
                    is_finished = True
                else:
                    if event.alt:
                        self.drag_point_id = self.find_point(mouse_pos_exact)
                        self.drag_mode = 'WEIGHT'
                    elif event.ctrl:
                        if self.enable_adding_points:
                            if (self.mouse_path - self.curve_points[self.finalized_points-1]["2d"]).magnitude > self.merge_radius:
                                self.add_cursor_point_to_list(context)
                                self.drag_mode = 'MOVE'
                    else:
                        self.drag_point_id = self.find_point(mouse_pos_exact)
                        self.drag_mode = 'MOVE'
                    if self.drag_mode == 'WEIGHT': mouse_pos = mouse_pos_exact
                    self.cal_angle = 0
                    self.mouse_pos = mouse_pos
                    self.drag_start = mouse_pos
                    self.drag_start_weight = self.point_weight()
                    self.drag_offset = self.point_2d() - mouse_pos
                    self.lmb_click_time = time.clock()
            else: # RELEASE
                if self.finalized_points == 2:
                    if (self.curve_points[0]["2d"] - self.curve_points[1]["2d"]).magnitude <= self.merge_radius:
                        self.finalized_points = 1
                        self.curve_points = self.curve_points[:-1]
                        self.update_point_list(context)
                if self.drag_mode == 'MOVE':
                    if (time.clock() - self.lmb_click_time) <= double_click_time * 0.35:
                        self.move_point(context, self.mouse_pos - self.drag_start)
                    self.draw_axis = False
                    self.snap_to_axis = False
                self.drag_mode = None

        elif event.type == 'MIDDLEMOUSE':
            if event.value == 'PRESS':
                if (time.clock() - self.mmb_click_time) <= double_click_time:
                    self.cut_through = True
                    self.use_3d_cursor = True
                    self.update_point_list(context)
                    if event.shift:
                        self.remove_cutters = False
                    is_finished = True
                else:
                    self.drag_point_id = self.find_point(mouse_pos_exact)
                    self.drag_mode = 'WEIGHT'
                    mouse_pos = mouse_pos_exact
                    self.mouse_pos = mouse_pos
                    self.drag_start = mouse_pos
                    self.drag_start_weight = self.point_weight()
                    self.drag_offset = self.point_2d() - mouse_pos
                    self.mmb_click_time = time.clock()
            else: # RELEASE
                self.drag_mode = None

        elif event.type == 'BACK_SPACE':
            if event.value == 'PRESS':
                if len(self.curve_points) > 1:
                    if len(self.curve_points) > self.finalized_points:
                        if self.finalized_points > 1:
                            self.curve_points.pop(-2)
                            self.finalized_points -= 1
                            self.drag_point_id -= 1
                    else:
                        self.curve_points.pop(-1)
                        self.finalized_points -= 1
                        self.drag_point_id -= 1
                    self.update_point_list(context)
        elif event.type == 'DEL':
            if event.value == 'PRESS':
                if (self.drag_mode is None) and (len(self.curve_points) > 1):
                    if len(self.curve_points) == self.finalized_points:
                        point_id = self.find_point(mouse_pos_exact)
                        if point_id >= 0:
                            self.curve_points.pop(point_id)
                            self.finalized_points -= 1
                            self.drag_point_id -= 1
                            self.update_point_list(context)
        elif event.type == 'X':
            if event.value == 'PRESS':
                if self.editmode:
                    if event.value == 'PRESS':
                        if self.mode != "SLICE":
                            self.mode = "SLICE"
                        else:
                            self.mode = "KNIFE"
                else:
                    if self.mode != "SLICE":
                        self.mode = "SLICE"
                    else:
                        self.mode = "CUT"
        elif event.type == 'A':
            if event.value == 'PRESS':
                if self.editmode:
                    if event.value == 'PRESS':
                        if self.mode != "MAKE":
                            self.mode = "MAKE"
                        else:
                            self.mode = "KNIFE"
                else:
                    if self.mode != "MAKE":
                        self.mode = "MAKE"
                    else:
                        self.mode = "CUT"
        elif event.type in {'MINUS', 'NUMPAD_MINUS', 'WHEELDOWNMOUSE'}:
            if event.value == 'PRESS':
                preference().BoxCutter_curve_vert_count -= 1
                self.update_point_list(context)
        elif event.type in {'EQUAL', 'NUMPAD_PLUS', 'WHEELUPMOUSE'}:
            if event.value == 'PRESS':
                preference().BoxCutter_curve_vert_count += 1
                self.update_point_list(context)
        elif event.type == 'Z':
            if event.value == 'PRESS':
                if event.alt:
                    self.snap_angle = not self.snap_angle
                elif event.shift:
                    preference().BoxCutter_enable_grid = not preference().BoxCutter_enable_grid
                else:
                    if self.editmode:
                        if event.value == 'PRESS':
                            if self.mode != "CUT":
                                self.mode = "CUT"
                            else:
                                self.mode = "KNIFE"
                    else:
                        if self.mode != "PANEL":
                            self.mode = "PANEL"
                        else:
                            self.mode = "CUT"
        elif event.type == 'Q':
            if event.value == 'PRESS':
                self.round_corners = not self.round_corners
        elif event.type == 'B':
            if event.value == 'PRESS':
                self.use_bevel_offset = not self.use_bevel_offset
        elif event.type == 'C':
            if event.value == 'PRESS':
                self.cut_through = not self.cut_through
                self.use_3d_cursor = not self.use_3d_cursor
        elif event.type == 'R':
            if event.value == 'PRESS':
                if self.drag_mode == 'ROTATE':
                    self.drag_mode = None
                else:
                    all_polygon_points = []
                    for p in self.curve_points:
                        all_polygon_points.append(p["2d"])
                    self.start_center = Vector((centroid(all_polygon_points)))
                    self.drag_start = mouse_pos_exact
                    self.drag_static = mouse_pos_exact
                    self.cal_angle = 0
                    self.drag_mode = 'ROTATE'
        elif event.type == 'S':
            if event.value == 'PRESS':
                if self.drag_mode == 'SCALE':
                    self.drag_mode = None
                else:
                    all_polygon_points = []
                    for p in self.curve_points:
                        all_polygon_points.append(p["2d"])
                    self.start_center = Vector((centroid(all_polygon_points)))
                    self.drag_start = mouse_pos_exact
                    self.drag_mode = 'SCALE'
        elif event.type == 'W':
            if event.value == 'PRESS':
                if event.shift:
                    self.use_x_axis = not self.use_x_axis
                else:
                    self.wire = not self.wire
                    show_obj_wires(context, self.wire)
        elif event.type == 'ONE':
            if event.value == 'PRESS':
                preference().BoxCutter_mirror_axis_x = not preference().BoxCutter_mirror_axis_x
        elif event.type == 'TWO':
            if event.value == 'PRESS':
                preference().BoxCutter_mirror_axis_y = not preference().BoxCutter_mirror_axis_y

        if is_finished or is_cancelled:
            preference().boxcutter_mirror_clump = False
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            preference().BoxCutter_help_item = 'MAIN'
            boxcutter_show_wire(self, context, False, self.wire)
            if BoxCutterDrawCurve.running_boxcutters.get(self._region) is self:
                del BoxCutterDrawCurve.running_boxcutters[self._region]
            if is_finished:
                self.execute(context)
                return {'FINISHED'}
            else:
                boxcutter_show_wire(self, context, False, self.wire)
                return {'CANCELLED'}

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        preference().BoxCutter_help_item = 'NGON'
        if preference().boxcutter_mirror_clump:
            self.clump_mirror = True

        self.use_3d_cursor = False
        self.running_boxcutters = []

        if context.area.type == 'VIEW_3D':

            removeable = []
            for live_name in live:
                shape = False
                for object in bpy.data.objects:
                    if live_name in bpy.data.objects and object == bpy.data.objects[live_name]:
                        shape = True
                if not shape:
                    removeable.append(live_name)
            for live_name in removeable:
                live.remove(live_name)

            if preference().BoxCutter_enter_ortho:
                set_viewport_ortho(context)
            self.lmb_click_time = time.clock()
            vx, vy, vz = context_cursor(context)
            self.cursor3d_pos = Vector((vx, vy, vz))
            cursor3d_follow(self, context)
            self.first_active = bpy.context.scene.objects.active
            self.first_selected_objects = bpy.context.selected_objects
            args = (self, context)
            if context.active_object is not None:
                if context.active_object.mode == 'EDIT':
                    self.editmode = True
                self.wire = bpy.context.object.show_wire
            else:
                self.wire = None
            self._region_mouse = [context.region]
            self._region = context.region
            BoxCutterDrawCurve.running_boxcutters[self._region] = self
            boxcutter_show_wire(self, context, True, self.wire)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_curve, args, 'WINDOW', 'POST_PIXEL')

            grid_path_x, grid_path_y = set_grid_path(self, context, event)
            self.mouse_path = Vector((grid_path_x, grid_path_y))

            self.list_location_3d = []
            self.list_location_3d_mx = []
            self.list_location_3d_my = []
            self.list_location_3d_mxy = []
            set_evet_region(self, event, True)
            context.window_manager.modal_handler_add(self)

            self.curve_points = []
            self.finalized_points = 0
            self.draw_axis = False

            if self.use_saved_points: # used if operators is called from box/circle and wants to use defined points
                self.add_saved_points_to_list(context, shape=self.start_shape)
            else:
                self.add_cursor_point_to_list(context)
                if preference().BC_start_curve_from_line:
                    self.add_cursor_point_to_list(context) # immediately draw a line

            mouse_pos = Vector((grid_path_x, grid_path_y))

            self.drag_mode = 'MOVE'
            self.mouse_pos = mouse_pos
            self.drag_start = mouse_pos
            self.drag_start_weight = 0.0
            self.drag_offset = self.point_2d() - mouse_pos
            self.drag_point_id = 0 # for starting with a point
            if preference().BC_start_curve_from_line:
                self.drag_point_id = 1 # for starting with a line

            self.draw_points = False
            self.draw_outline = False
            self.draw_snap_angle = False

            self.snap_angle_pos = Vector((0, 0))
            self.snap_angle_dir = Vector((0, 0))

            self.x = event.mouse_region_x
            self.y = event.mouse_region_y
            x_abs = self.x + context.region.x
            y_abs = self.y + context.region.y

            ui_contexts = list(ui_contexts_under_coord(x_abs, y_abs, context.window))
            if ui_contexts and ui_contexts[0]:
                self._region_mouse = [uic["region"] for uic in ui_contexts]

            preference().BoxCutter_curve_vert_count = preference().BoxCutter_curve_vert_count
            self.angle_subdivs = 360.0/preference().BoxCutter_snapping_angle

            return {'RUNNING_MODAL'}
        else:
            return {'CANCELLED'}

    def is_on_side(self, context, point):
            all_polygon_points = []
            for p in self.curve_points:
                all_polygon_points.append(p["2d"])
            pc = Vector((centroid(all_polygon_points)))
            p1 = Vector((pc.x+1, pc.y+1))
            p2 = Vector((pc.x-1, pc.y-1))
            p3 = Vector((pc.x+1, pc.y-1))
            p4 = Vector((pc.x-1, pc.y+1))
            v1 = Vector((p2.x-p1.x, p2.y-p1.y, 0))
            v2 = Vector((p2.x-point.x, p2.y-point.y, 0))
            v3 = Vector((p4.x-p3.x, p4.y-p3.y, 0))
            v4 = Vector((p4.x-point.x, p4.y-point.y, 0))
            xp1 = cross(v1, v2)
            xp2 = cross(v3, v4)

            if xp1 > 0 and xp2 < 0:
                return 'LEFT'
            elif xp1 < 0 and xp2 > 0:
                return 'RIGHT'
            elif xp1 > 0 and xp2 > 0:
                return 'TOP'
            elif xp1 < 0 and xp2 < 0:
                return 'BOTTOM'
            else:
                print('on the same line!')

    def find_point(self, pos_2d):
        r = self.point_radius
        for i in range(len(self.curve_points)):
            p = self.curve_points[i]
            d = p["2d"] - pos_2d
            if max(abs(d.x), abs(d.y)) < r:
                return i
        return -1

    def point_2d(self, i=None):
        if i is None: i = self.drag_point_id
        if (i < 0) or (i >= len(self.curve_points)): return Vector((0, 0))
        return self.curve_points[self.drag_point_id]["2d"]

    def point_3d(self, i=None):
        if i is None: i = self.drag_point_id
        if (i < 0) or (i >= len(self.curve_points)): return Vector((0, 0, 0))
        return self.curve_points[self.drag_point_id]["3d"]

    def point_weight(self, i=None):
        if i is None: i = self.drag_point_id
        if (i < 0) or (i >= len(self.curve_points)): return 0.0
        return self.curve_points[self.drag_point_id]["weight"]

    def set_point_2d(self, context, pos2d, i=None):
        if i is None: i = self.drag_point_id
        if i < 0: return
        if self.snap_angle and (len(self.curve_points) > 1):
            pos2d_prev = self.curve_points[(self.drag_point_id-1) % len(self.curve_points)]["2d"]
            angle0 = 0.0
            if (len(self.curve_points) > 2):
                pos2d_prev2 = self.curve_points[(self.drag_point_id-2) % len(self.curve_points)]["2d"]
                angle0 = (pos2d_prev - pos2d_prev2).angle_signed(Vector((1, 0)), 0.0)
            delta = pos2d - pos2d_prev
            angle = delta.angle_signed(Vector((1, 0)), 0.0)
            angle_grid = (math.pi*2)/self.angle_subdivs
            angle = round((angle - angle0)/angle_grid)*angle_grid + angle0
            new_dir = Vector((math.cos(angle), math.sin(angle)))
            pos2d = pos2d_prev + delta.project(new_dir)
            self.snap_angle_pos = Vector(pos2d_prev)
            self.snap_angle_dir = new_dir
        else:
            self.snap_angle_pos = Vector((0, 0))
            self.snap_angle_dir = Vector((0, 0))
        p = self.curve_points[self.drag_point_id]
        p["2d"] = pos2d
        self.update_point_list(context)

    def move_point_onaxis(self, context, delta, axis='y', i=None):
        if i is None: i = self.drag_point_id
        if i < 0:
            for p in self.curve_points:
                if axis is 'y':
                    p["2d"].y = p["2d"].y + delta
                elif axis is 'x':
                    p["2d"].x = p["2d"].x + delta
        else:
            p = self.curve_points[self.drag_point_id]
            if axis is 'y':
                p["2d"].y = p["2d"].y + delta
            elif axis is 'x':
                p["2d"].x = p["2d"].x + delta
        self.update_point_list(context)

    def move_point(self, context, delta, i=None):
        if i is None: i = self.drag_point_id
        if i < 0:
            for p in self.curve_points:
                p["2d"] = p["2d"] + delta
        else:
            try: p = self.curve_points[self.drag_point_id]
            except: return {'CANCELLED'}
            p["2d"] = p["2d"] + delta
        self.update_point_list(context)

    def scale_points(self, context, scale):
        all_polygon_points = []
        for p in self.curve_points:
            all_polygon_points.append(p["2d"])
        for p in self.curve_points:
            p["2d"] = p["2d"] - Vector((centroid(all_polygon_points)))
            p["2d"] *= scale
            p["2d"] = p["2d"] + Vector((centroid(all_polygon_points)))

        self.update_point_list(context)

    def rotate_point(self, context, angle):
        all_polygon_points = []
        for p in self.curve_points:
            all_polygon_points.append(p["2d"])
        for p in self.curve_points:
            p["2d"] = Vector((rotate(centroid(all_polygon_points), p["2d"], angle)))

        self.update_point_list(context)

    def clamp_weight(self, weight):
        if self.round_corners:
            return max(weight, 0.0)
        else:
            return min(max(weight, 0.0), 1.0)

    def set_point_weight(self, context, weight, i=None):
        if i is None: i = self.drag_point_id
        if i < 0: return
        p = self.curve_points[self.drag_point_id]
        p["weight"] = self.clamp_weight(weight)
        self.update_point_list(context)

    def add_point_weight(self, context, delta_weight, i=None):
        if i is None: i = self.drag_point_id
        if i < 0:
            for p in self.curve_points:
                p["weight"] = self.clamp_weight(p["weight"]+delta_weight)
        else:
            p = self.curve_points[self.drag_point_id]
            p["weight"] = self.clamp_weight(p["weight"]+delta_weight)
        self.update_point_list(context)

    def interpolate_curve_segment(self, i, resolution):
        n_points = len(self.curve_points)
        p0h = self.curve_points[(i-1) % n_points]
        p0 = self.curve_points[i]
        p1 = self.curve_points[(i+1) % n_points]
        p1h = self.curve_points[(i+2) % n_points]

        weight1 = p0["weight"]
        weight2 = p1["weight"]
        knot1 = p0["2d"]
        knot2 = p1["2d"]

        if max(weight1, weight2) <= 1e-6:
            return [knot1, knot2]

        if self.round_corners:
            handle1 = p0h["2d"]
            handle2 = p1h["2d"]

            r1 = p0["r"]
            r2 = p1["r"]

            vertices = []

            n = resolution // 2

            if r1 <= 1e-6:
                vertices.append(p0["2d"])
            else:
                vertices.extend(self.interpolate_arc(handle1, knot1, knot2, r1, n))
                if n == 0: vertices = vertices[1:]

            if r2 <= 1e-6:
                vertices.append(p1["2d"])
            else:
                vertices.extend(self.interpolate_arc(handle2, knot2, knot1, r2, n, reverse=True))

            return vertices
        else:
            weight1 = min(max(weight1, 0.0), 1.0)
            weight2 = min(max(weight2, 0.0), 1.0)
            handle1 = knot1 + (knot2 - p0h["2d"]) * (weight1 * 0.5)
            handle2 = knot2 + (knot1 - p1h["2d"]) * (weight2 * 0.5)
            return interpolate_bezier(knot1, handle1, handle2, knot2, max(resolution, 2))

    def max_radius_angle(self, pL, pC, pR, l=None):
        dCL = (pL - pC)
        dCR = (pR - pC)
        max_len = min(dCL.magnitude, dCR.magnitude)
        if not (l is None): max_len = min(max_len, l)
        a2 = dCL.angle(dCR, 0.0) * 0.5
        r = math.tan(a2) * max_len
        return r, a2

    def interpolate_arc(self, pL, pC, pR, r, n, reverse=False):
        dCL = (pL - pC).normalized()
        dCR = (pR - pC).normalized()
        a2 = dCL.angle(dCR, 0.0) * 0.5
        dCM = (dCL + dCR).normalized()
        t = (0.0 if a2 < 1e-6 else (r / math.sin(a2)))
        dCM_ = dCM * t
        c = pC + dCM_ # center of the circle
        cp = dCM_.project(dCR) # projection on CR
        dR = cp - dCM_
        dM = -dCM * r
        aM = dM.angle_signed(Vector((1, 0)), 0.0)
        aR = dR.angle_signed(Vector((1, 0)), 0.0)
        da = aR - aM
        if (da > math.pi): da -= math.pi * 2
        if (da < -math.pi): da += math.pi * 2
        if n > 0:
            for i in range(n+1):
                t = float(i) / float(n)
                if reverse: t = 1.0 - t
                a = aM + da * t
                dr = Vector((r*math.cos(a), r*math.sin(a)))
                res = c + dr
                yield res
        else:
            cR = Vector((r*math.cos(aR), r*math.sin(aR)))
            cM = cR.project(dCM)
            if reverse:
                yield c + cR
                yield c + cM
            else:
                yield c + cM
                yield c + cR

    def clamp_radii(self):
        n_points = len(self.curve_points)
        for i in range(n_points):
            pL = self.curve_points[(i-1) % n_points]
            pC = self.curve_points[i]
            pR = self.curve_points[(i+1) % n_points]
            weight = max(pC["weight"], 0.0)
            l = (weight if self.use_bevel_offset else None)
            r, a2 = self.max_radius_angle(pL["2d"], pC["2d"], pR["2d"], l)
            if not self.use_bevel_offset: r = min(weight, r)
            l = (0.0 if a2 < 1e-6 else (r / math.tan(a2)))
            pC["a"] = a2
            pC["r"] = r
            pC["l"] = l
            pC["dl"] = 0.0

        for i in range(n_points):
            pA = self.curve_points[i]
            pB = self.curve_points[(i+1) % n_points]
            AB = (pB["2d"] - pA["2d"]).magnitude
            lAB = pA["l"] + pB["l"]
            if lAB <= AB: continue
            dl = (lAB - AB) * 0.5
            pA["dl"] = max(pA["dl"], dl)
            pB["dl"] = max(pB["dl"], dl)

        for p in self.curve_points:
            p["l"] = p["l"] - p["dl"]
            p["r"] = math.tan(p["a"]) * p["l"]

    def update_point_list(self, context):
        n_points = len(self.curve_points)
        if n_points <= 2:
            interpolated_2d = [p["2d"] for p in self.curve_points]
        else:
            self.clamp_radii()
            interpolated_2d = []
            for i in range(n_points):
                vertices = self.interpolate_curve_segment(i, preference().BoxCutter_curve_vert_count)
                for j in range(len(vertices)-1):
                    interpolated_2d.append(vertices[j])

        region = context.region
        rv3d = region3d(self, context)

        object = bpy.context.object
        if self.mode in {"CUT", "SLICE", "PANEL"}:
            local_bbox_center = 0.125 * sum((Vector(b) for b in object.bound_box), Vector())
            global_bbox_center = object.matrix_world * local_bbox_center

            if self.use_3d_cursor:
                self.depth_location = context_cursor(context)
            else:
                self.depth_location = global_bbox_center
        else:
            self.depth_location = context_cursor(context)

        self.list_location_3d = []
        v_prev = None
        for v in interpolated_2d:
            if not (v_prev is None):
                if (v-v_prev).magnitude <= self.merge_radius: continue
            v_3d = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, v, self.depth_location)
            self.list_location_3d.append(v_3d)
            v_prev = v

    def update_mirror_point_list(self, context):
        n_points = len(self.curve_points)
        if n_points <= 2:
            interpolated_2d = [p["2d"] for p in self.curve_points]
        else:
            self.clamp_radii()
            interpolated_2d = []
            for i in range(n_points):
                vertices = self.interpolate_curve_segment(i, preference().BoxCutter_curve_vert_count)
                for j in range(len(vertices)-1):
                    interpolated_2d.append(vertices[j])

        region = context.region
        rv3d = region3d(self, context)

        object = bpy.context.object
        if self.mode in {"CUT", "SLICE", "PANEL"}:
            local_bbox_center = 0.125 * sum((Vector(b) for b in object.bound_box), Vector())
            global_bbox_center = object.matrix_world * local_bbox_center

            if self.use_3d_cursor:
                self.depth_location = context_cursor(context)
            else:
                self.depth_location = global_bbox_center
        else:
            self.depth_location = context_cursor(context)

        self.list_location_3d_mx = []
        v_prev = None
        for v in interpolated_2d:
            if not (v_prev is None):
                if (v-v_prev).magnitude <= self.merge_radius: continue
            object = bpy.context.active_object
            if object is not None:
                if preference().BoxCutter_mirror_type == "3D":
                    origin = point_to_2d(self, context, self.first_active.location[0], self.first_active.location[1], self.first_active.location[2])
                else:
                    origin = point_to_2d(self, context, bpy.context.space_data.cursor_location[0], bpy.context.space_data.cursor_location[1], bpy.context.space_data.cursor_location[2])
            else:
                origin = point_to_2d(self, context, 0, 0, 0)
            vx = scale(origin, v, "X")
            v_3d_mx = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, vx, self.depth_location)
            vy = scale(origin, v, "Y")
            v_3d_my = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, vy, self.depth_location)
            vy = scale(origin, v, "XY")
            v_3d_mxy = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, vy, self.depth_location)
            self.list_location_3d_mx.append(v_3d_mx)
            self.list_location_3d_my.append(v_3d_my)
            self.list_location_3d_mxy.append(v_3d_mxy)
            v_prev = v

    def add_cursor_point_to_list(self, context, finalized=True):
        if len(self.curve_points) > self.finalized_points:
            if finalized:
                self.finalized_points += 1
            return

        self.curve_points.append({"2d": Vector(self.mouse_path), "3d": Vector(), "weight": 0.0, "a": 0.0, "r": 0.0, "l": 0.0, "dl": 0.0})
        self.drag_point_id = len(self.curve_points)-1

        if finalized:
            self.finalized_points = len(self.curve_points)

    def add_saved_points_to_list(self, context, finalized=True, shape="BOX"):
        if len(self.curve_points) > self.finalized_points:
            if finalized:
                self.finalized_points += 1
            return

        if shape == 'CIRCLE':
            for point in saved_circle_points:
                self.curve_points.append({"2d": Vector(point), "3d": Vector(), "weight": 0.0, "a": 0.0, "r": 0.0, "l": 0.0, "dl": 0.0})
        elif shape == 'BOX':
            for point in saved_points:
                self.curve_points.append({"2d": Vector(point), "3d": Vector(), "weight": 0.0, "a": 0.0, "r": 0.0, "l": 0.0, "dl": 0.0})

        self.drag_point_id = len(self.curve_points)-1
        self.update_point_list(context)

        if finalized:
            self.finalized_points = len(self.curve_points)
