import bpy
import bpy_extras.view3d_utils
from mathutils import Vector
import bmesh
from bpy.props import *
from ... utils.modifiers import live
from ... utils.objects import set_active, show_obj_wires
from ... utils.modifiers import create_solidify_modifier
from ... utils.blender_ui import region_dimensions
from ... utils.space_3d import center_point, context_cursor, set_context_cursor, cursor_2d_location, region3d
from ... preferences import preference
from .. boolean_operators.bool_bmesh import bc_diff_boolean


saved_cords = [0, 0]
enable_grid = False


def view_directions(self, context):
    region = context.region
    rv3d = region3d(self, context)
    v0 = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, Vector((0, 0)), Vector())
    vX = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, Vector((1, 0)), Vector())
    vY = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, Vector((0, 1)), Vector())
    dirX = (vX - v0).normalized()
    dirY = (vY - v0).normalized()
    dirZ = dirX.cross(dirY)
    return (dirX, dirY, dirZ)


def generate_3d_mesh(self, context, bmesh_matrix):
    view_direction = view_directions(self, context)[2]

    selection = bpy.context.scene.objects.active

    saved_cursor = [0, 0, 0]
    saved_cursor[0], saved_cursor[1], saved_cursor[2] = context_cursor(context)

    make_bmesh(self, context)
    set_context_cursor(context, Vector((saved_cursor)))
    object = bpy.context.active_object

    if self.mode in {"KNIFE"}:
        obj = bpy.context.object
        dimensions = [abs(self.first_active.dimensions[0]), abs(self.first_active.dimensions[1]), abs(self.first_active.dimensions[2])]
        biggest_dimension = max(dimensions)

        vec = Vector((0.0, 0.0, biggest_dimension))
        inv = obj.matrix_world.copy()
        inv.invert()

        vec_rot = vec * inv
        obj.location = obj.location + vec_rot

    if self.mode in {"CUT"}:
        selection.select = True
        thickess = preference().BoxCutter_box_depth
        cutter = bpy.context.scene.objects.active
        self.cutter_obj = cutter
        # bpy.context.scene.objects.active = self.first_active
        bpy.ops.object.select_all(action='DESELECT')
        cutter.select = True
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY')
        biggest_dimension = [abs(self.first_active.dimensions[0]), abs(self.first_active.dimensions[1]), abs(self.first_active.dimensions[2])]
        thickess = max(biggest_dimension)
        if self.use_3d_cursor:
            if preference().BoxCutter_cut_depth > 0:
                thickess = preference().BoxCutter_cut_depth
        set_active(cutter, True, True)
        bpy.ops.view3d.snap_cursor_to_selected()
        bpy.ops.object.origin_set(type='ORIGIN_CURSOR')

        set_active(self.first_active, True, True)
        set_active(cutter, True)

        create_solidify_modifier(cutter, thickess*2, 0, True, False, True, True)

        bpy.ops.object.modifier_apply(apply_as="DATA", modifier='Solidify')

    if self.mode in {"SLICE"}:
        selection.select = True
        thickess = preference().BoxCutter_box_depth
        cutter = bpy.context.scene.objects.active
        self.cutter_obj = cutter
        # bpy.context.scene.objects.active = self.first_active
        bpy.ops.object.select_all(action='DESELECT')
        cutter.select = True
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY')
        biggest_dimension = [abs(self.first_active.dimensions[0]), abs(self.first_active.dimensions[1]), abs(self.first_active.dimensions[2])]
        thickess = max(biggest_dimension)
        if self.use_3d_cursor:
            if preference().BoxCutter_cut_depth > 0:
                thickess = preference().BoxCutter_cut_depth
        set_active(self.first_active, True, True)
        set_active(cutter, True)
        # object = bpy.context.active_object
        create_solidify_modifier(cutter, thickess*2, 0, True, False, True, True)

        bpy.ops.object.modifier_apply(apply_as="DATA", modifier='Solidify')

    if self.mode in {"MAKE"}:
        thickess = preference().BoxCutter_box_depth
        cutter = bpy.context.scene.objects.active
        self.cutter_obj = cutter
        # bpy.context.scene.objects.active = self.first_active

        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY')

        object = bpy.context.active_object

        set_active(cutter, True, True)
        object = bpy.context.active_object
        set_context_cursor(context, Vector((object.location)))
        if preference().BoxCutter_box_depth > 0:
            create_solidify_modifier(object, thickess*2, 0, True, False, True, True)
            bpy.ops.object.modifier_apply(apply_as="DATA", modifier='Solidify')

    elif self.mode in {"PANEL"}:
        thickess = preference().BoxCutter_box_depth
        cutter = bpy.context.scene.objects.active
        self.cutter_obj = cutter
        # bpy.context.scene.objects.active = self.first_active
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY')
        set_active(self.first_active, True, True)
        set_active(cutter, True)
        object = bpy.context.active_object

    if self.use_3d_cursor:
        if self.mode not in {"KNIFE"}:
            object.location += view_direction * thickess
        if self.mode in {"CUT", "MAKE"}:
            set_active(object, True, True)
            bpy.ops.object.origin_set(type='ORIGIN_CURSOR')
            bpy.ops.transform.translate(value=(0, 0, -preference().BoxCutter_cut_depth), constraint_axis=(False, False, True), constraint_orientation='VIEW')

    set_context_cursor(context, Vector((saved_cursor)))


def make_bmesh(self, context):

    set_context_cursor(context, Vector((0, 0, 0)))

    bpy.ops.object.add(type='MESH', view_align=False, enter_editmode=False)

    ob = bpy.context.object
    me = ob.data

    bm = bmesh.new()
    list_verts = []
    list_verts_mirror_x = []
    list_verts_mirror_y = []
    list_verts_mirror_xy = []

    for v in self.list_location_3d:
        list_verts.append(bm.verts.new(v))
    try:
        bm.faces.new(list_verts)
    except:
        pass

    if not self.clump_mirror:
        if preference().BoxCutter_mirror_axis_x:
            for v in self.list_location_3d_mx:
                list_verts_mirror_x.append(bm.verts.new(v))
            try:
                bm.faces.new(list_verts_mirror_x)
            except:
                pass

        if preference().BoxCutter_mirror_axis_y:
            for v in self.list_location_3d_my:
                list_verts_mirror_y.append(bm.verts.new(v))
            try:
                bm.faces.new(list_verts_mirror_y)
            except:
                pass

        if preference().BoxCutter_mirror_axis_x and preference().BoxCutter_mirror_axis_y:
            for v in self.list_location_3d_mxy:
                list_verts_mirror_xy.append(bm.verts.new(v))
            try:
                bm.faces.new(list_verts_mirror_xy)
            except:
                pass

    for v in bm.verts:
        v.select = True

    bm.to_mesh(me)
    bm.free()

    oldselection = bpy.context.active_object
    bpy.ops.object.add(type='MESH', view_align=True, enter_editmode=False)

    ob = bpy.context.object

    if self.mode == "KNIFE":
        ob.name = 'BC_KNFE_CUTTER'
    else:
        ob.name = 'BC-cutter'

    if not self.remove_cutters:
        if ob.name in live:
            ob.name += '.{0:03d}'.format(len([name for name in live if name == ob.name or name[:-4] == ob.name]))
        else:
            ob.name += '.000'

        live.append(ob.name)

    oldselection.select = True
    bpy.ops.object.join()


def append_vert_to_list(self, context, x, y):
    object = bpy.context.object
    region = context.region
    rv3d = region3d(self, context)
    if self.mode in {"CUT", "SLICE", "PANEL"}:
        local_bbox_center = 0.125 * sum((Vector(b) for b in object.bound_box), Vector())
        global_bbox_center = object.matrix_world * local_bbox_center

        if self.use_3d_cursor:
            depth_location = context_cursor(context)
        else:
            depth_location = global_bbox_center
    else:
        depth_location = context_cursor(context)

    region_2d = Vector((x, y))
    location_3d = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, region_2d, depth_location)
    self.list_location_3d.append(location_3d)
    depth_location = self.list_location_3d[0]


def set_evet_region(self, event, first):
    if first:
        self.first_mouse_x = self.mouse_path[0]
        self.first_mouse_y = self.mouse_path[1]
    else:
        self.last_mouse_region_x = self.mouse_path[0]
        self.last_mouse_region_y = self.mouse_path[1]


def set_saved_region(self, event):
    self.first_mouse_x = self.mouse_path[0]+self.savedx
    self.first_mouse_y = self.mouse_path[1]+self.savedy


def check_if_moving(self, context, event):
    if preference().BoxCutter_shape_moving is True:
        set_saved_region(self, event)


def boxcutter_show_wire(self, context, enable, is_off):
    if preference().BoxCutter_show_wires is True:
        if self.first_active in list(bpy.context.selected_objects):
            if not is_off:
                show_obj_wires(context, enable)


def set_grid_path(self, context, event):

    region_height, region_width = region_dimensions()
    center = center_point(self, context)
    grid_path_spaceing = preference().BoxCutter_grid_spaceing // context.space_data.region_3d.view_distance

    if preference().BoxCutter_enable_grid:
        if preference().BoxCutter_type_of_grid == "DEFAULT":

            cursor_2d = cursor_2d_location(self, context)

            grid_path_x = cursor_2d[0]
            grid_path_y = cursor_2d[1]

            set_context_cursor(context, self.cursor3d_pos)

        elif preference().BoxCutter_type_of_grid == "CUSTOM":

            grid_path_x = int(int(center[0]) - (grid_path_spaceing * (int((center[0] - event.mouse_region_x)/grid_path_spaceing))))
            grid_path_y = int(int(center[1]) - (grid_path_spaceing * (int((center[1] - event.mouse_region_y)/grid_path_spaceing))))

    else:
        grid_path_x = event.mouse_region_x
        grid_path_y = event.mouse_region_y

    return grid_path_x, grid_path_y


def project_make_mesh(self, context, cut_through):
    bpy.ops.object.mode_set(mode='EDIT')
    bpy.ops.mesh.reveal()
    bpy.ops.mesh.knife_project(cut_through=cut_through)
    bpy.ops.object.mode_set(mode='OBJECT')
    bpy.ops.object.join()
    bpy.ops.object.mode_set(mode='EDIT')
    bpy.ops.mesh.remove_doubles()


def project_knife_mesh(self, context, cut_through):
    bpy.ops.object.mode_set(mode='EDIT')
    if self.knife_selection:
        bpy.ops.mesh.hide(unselected=True)
    bpy.ops.mesh.knife_project(cut_through=cut_through)
    if self.knife_selection:
        bpy.ops.mesh.reveal()
    objs = bpy.data.objects
    objs.remove(objs["BC_KNFE_CUTTER"], True)


def generate_panel_mesh(self, context):

    self.cutter_obj.select = False
    bpy.ops.object.duplicate()
    self.first_active.select = False

    duplicate_obj = bpy.context.active_object
    self.cutter_obj.select = True
    # bpy.context.scene.objects.active = duplicate_obj

    duplicate_obj.modifiers.clear()

    bpy.ops.object.editmode_toggle()
    bpy.ops.mesh.select_mode(type="FACE")
    bpy.ops.mesh.select_all(action='DESELECT')
    bpy.ops.mesh.knife_project(cut_through=True)
    bpy.ops.mesh.select_all(action='INVERT')
    bpy.ops.mesh.delete(type='FACE')
    bpy.ops.object.editmode_toggle()
    bpy.context.object.draw_type = 'WIRE'

    solidify_modifier = duplicate_obj.modifiers.new("Solidify", "SOLIDIFY")
    solidify_modifier.thickness = 0.100
    solidify_modifier.offset = 0
    solidify_modifier.use_even_offset = True
    solidify_modifier.use_quality_normals = True
    solidify_modifier.use_rim_only = False
    solidify_modifier.show_on_cage = True

    duplicate_obj.select = False
    bpy.ops.object.delete(use_global=False)

    duplicate_obj.select = True
    self.first_active.select = True
    # bpy.context.scene.objects.active = self.first_active
    bc_diff_boolean(context, False, False, False)
    # bpy.ops.boxcutter.bool_difference(remove_cutters=False, apply_cutters=False)

    duplicate_obj.select = True
    self.first_active.select = False
    bpy.context.scene.objects.active = duplicate_obj


def cursor3d_follow(self, context):
    if preference().BoxCutter_enable_grid:
        if preference().BoxCutter_type_of_grid == "DEFAULT":
            bpy.ops.view3d.cursor3d('INVOKE_DEFAULT')
            bpy.ops.view3d.snap_cursor_to_grid()
