import bpy
import bmesh
from mathutils import Vector


class BoxCutterDrawViewAlign(bpy.types.Operator):
        bl_idname = "boxcutter.view_align"
        bl_label = "Box Cutter View Align"
        bl_options = {'REGISTER', 'UNDO'}

        editmode = True
        mouse_path = Vector((0, 0))
        face_origin = Vector((0, 0, 0))
        face_index = int()

        @classmethod
        def poll(cls, context):
            return context.area.type == "VIEW_3D" and context.object and context.object.type == 'MESH'

        def cancel(self, context):
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, "WINDOW")

        def execute(self, context):
            mesh = context.object.data
            object_mode = context.object.mode

            if object_mode != "EDIT":
                self.editmode = False
                bpy.ops.object.editmode_toggle()
            else:
                self.editmode = True

            selection_modes = context.scene.tool_settings.mesh_select_mode[:]
            context.scene.tool_settings.mesh_select_mode = (False, False, True)

            bm = bmesh.from_edit_mesh(mesh)

            hidden_geometry = {'verts' : [vert.hide for vert in bm.verts],
                               'edges' : [edge.hide for edge in bm.edges],
                               'faces' : [face.hide for face in bm.faces]}

            selected_geometry = {'verts': [vert.select for vert in bm.verts],
                                 'edges': [edge.select for edge in bm.edges],
                                 'faces': [face.select for face in bm.faces]}

            mouse_position = int(self.mouse_path[0]), int(self.mouse_path[1])

            if not self.editmode:
                for hide_type in hidden_geometry:
                    bm_geometry = getattr(bm, hide_type)
                    for geo in bm_geometry:
                        geo.hide_set(False)

            is_geometry = bpy.ops.view3d.select(location=mouse_position) != {'PASS_THROUGH'}

            context.scene.tool_settings.mesh_select_mode = selection_modes

            if is_geometry:
                bpy.ops.view3d.viewnumpad(type="TOP", align_active=True)

                if not self.editmode:
                    for hide_type in hidden_geometry:
                        bm_geometry = getattr(bm, hide_type)
                        for hidden, bm_geometry in zip(hidden_geometry[hide_type], bm_geometry):
                            bm_geometry.hide_set(hidden)

                for selection_type in selected_geometry:
                    bm_geometry = getattr(bm, selection_type)
                    for selected, bm_geometry in zip(selected_geometry[selection_type], bm_geometry):
                        bm_geometry.select_set(selected)

            else:
                bpy.ops.object.mode_set(mode=object_mode)
                return {'CANCELLED'}

            if object_mode != "EDIT":
                bpy.ops.object.mode_set(mode=object_mode)

            return {'FINISHED'}

        def invoke(self, context, event):

            self.mouse_path = Vector((event.mouse_region_x, event.mouse_region_y))
            self.execute(context)

            return {'FINISHED'}
