import bpy
from bgl import *
import bmesh
import bpy_extras.view3d_utils
from mathutils import Vector
from bpy.props import *
from .. draw_operators.draw_utils import set_evet_region, set_grid_path
from ... utils.space_3d import context_cursor, region3d
from ... utils.view_3d import intersect


class BoxCutterMakeEdge(bpy.types.Operator):
        bl_idname = "boxcutter.make_edge"
        bl_label = "Box Cutter Make Edge"
        bl_options = {'REGISTER', 'UNDO'}

        mouse_path = Vector((0, 0))

        def draw(self, context):
            layout = self.layout
            box = layout.box()
            col = box.column()
            col.label("Box Cutter")

        def execute(self, context):
            create_edge(self, context)
            return {'FINISHED'}

        def cancel(self, context):
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')

        def modal(self, context, event):
            context.area.tag_redraw()

            if event.type == 'MOUSEMOVE':
                grid_path_x, grid_path_y = set_grid_path(self, context, event)
                self.mouse_path = Vector((event.mouse_region_x, event.mouse_region_y))

            elif event.type == 'LEFTMOUSE' and event.value == 'RELEASE' and not event.alt:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                self.execute(context)
                return {'FINISHED'}

            elif event.type == 'RIGHTMOUSE':
                self.cancel(context)
                return {'CANCELLED'}

            elif event.type in {'ESC'}:
                self.cancel(context)
                return {'CANCELLED'}

            return {'RUNNING_MODAL'}

        def invoke(self, context, event):

            if context.area.type == 'VIEW_3D':
                self.first_active = bpy.context.scene.objects.active
                args = (self, context)
                self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_line, args, 'WINDOW', 'POST_PIXEL')
                grid_path_x, grid_path_y = set_grid_path(self, context, event)
                self.mouse_path = Vector((grid_path_x, grid_path_y))
                self.first_mouse = self.mouse_path
                set_evet_region(self, event, True)
                context.window_manager.modal_handler_add(self)
                return {'RUNNING_MODAL'}
            else:
                self.report({'WARNING'}, "View3D not found, cannot run operator")
                return {'CANCELLED'}


def draw_line(self, context):

    dx1 = self.mouse_path[0]
    dy1 = self.mouse_path[1]

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)

    glColor4f(0.35, 1, 0.29, 0.53)
    glLineWidth(2.5)

    glBegin(GL_LINES)
    glVertex2f(self.first_mouse[0], self.first_mouse[1])
    glVertex2f(dx1, dy1)
    glEnd()

    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)


def create_edge(self, context):

    region = context.region
    rv3d = region3d(self, context)
    depth_location = context_cursor(context)
    ob = bpy.context.object
    me = ob.data
    bm = bmesh.from_edit_mesh(me)
    mat = ob.matrix_world

    selected_verts = [vert for vert in bm.verts if vert.select]

    v1_2d = bpy_extras.view3d_utils.location_3d_to_region_2d(bpy.context.region, rv3d, Vector((mat * selected_verts[0].co)))
    v2_2d = bpy_extras.view3d_utils.location_3d_to_region_2d(bpy.context.region, rv3d, Vector((mat * selected_verts[1].co)))

    if_intersect = intersect(v1_2d, self.first_mouse, v2_2d, self.mouse_path)

    if if_intersect:
        depth_location = mat * selected_verts[1].co
    else:
        depth_location = mat * selected_verts[0].co

    v1_3d = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, self.first_mouse, depth_location)

    if if_intersect:
        depth_location = mat * selected_verts[0].co
    else:
        depth_location = mat * selected_verts[1].co

    v2_3d = bpy_extras.view3d_utils.region_2d_to_location_3d(region, rv3d, self.mouse_path, depth_location)

    v1 = bm.verts.new(mat.inverted() * v1_3d)
    v2 = bm.verts.new(mat.inverted() * v2_3d)

    new_edge = bm.edges.new((v1, v2))
    new_edge.select = True
    bpy.ops.mesh.bridge_edge_loops()
    bpy.ops.mesh.select_all(action='DESELECT')
    new_edge.select = True

    bmesh.update_edit_mesh(me)
