import bpy
from bgl import *
import math
import bpy_extras.view3d_utils
from mathutils import Vector
from bpy.props import *

modes = [
    ("CUT", "Cut", ""),
    ("SLICE", "Slice", ""),
    ("MAKE", "Make", ""),
    ("KNIFE", "Knife", ""),
    ("PANEL", "Panel", "")]


class BoxCutterInvokeOperators(bpy.types.Operator):
        bl_idname = "boxcutter.invoke_operators"
        bl_label = "Box Cutter Invoke Operators"
        bl_options = {'REGISTER', 'UNDO'}

        mode = EnumProperty(name="", default="CUT", options={"SKIP_SAVE"}, items=modes)

        def draw(self, context):
            layout = self.layout
            box = layout.box()
            col = box.column()
            col.label("Box Cutter")

        def execute(self, context):
            if self.mode == "SLICE":
                bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="SLICE")
            elif self.mode == "CUT":
                bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT')
            elif self.mode == "MAKE":
                bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="MAKE")
            elif self.mode == "PANEL":
                bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="PANEL")
            elif self.mode == "KNIFE":
                bpy.ops.boxcutter.draw_poly('INVOKE_DEFAULT', mode="KNIFE")
            return {'FINISHED'}

        def cancel(self, context):
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')

        def modal(self, context, event):
            context.area.tag_redraw()

            if event.type == 'MOUSEMOVE':
                self.mouse_path = Vector((event.mouse_region_x, event.mouse_region_y))

            elif event.type == 'LEFTMOUSE' and event.value == 'PRESS':
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                self.execute(context)
                return {'FINISHED'}

            elif event.type == 'RIGHTMOUSE':
                self.cancel(context)
                return {'CANCELLED'}

            elif event.type in {'ESC'}:
                self.cancel(context)
                return {'CANCELLED'}

            return {'RUNNING_MODAL'}

        def invoke(self, context, event):

            if context.area.type == 'VIEW_3D':
                self.first_active = bpy.context.scene.objects.active
                args = (self, context)
                self.mouse_path = Vector((event.mouse_region_x, event.mouse_region_y))
                self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_cursor, args, 'WINDOW', 'POST_PIXEL')
                context.window_manager.modal_handler_add(self)
                return {'RUNNING_MODAL'}
            else:
                self.report({'WARNING'}, "View3D not found, cannot run operator")
                return {'CANCELLED'}


def draw_cursor(self, context):

    center = self.mouse_path

    if self.mode == "SLICE":
        glColor4f(0.62, 0.5, 0.2, 0.5)
    elif self.mode == "CUT":
        glColor4f(0.6, 0.2, 0.2, 0.5)
    elif self.mode == "MAKE":
        glColor4f(0.5, 0.5, 0.5, 0.5)
    elif self.mode == "PANEL":
        glColor4f(0.4, 0.2, 0.8, 0.5)
    elif self.mode == "KNIFE":
        glColor4f(0.29, 0.52, 1.0, 0.53)

    radius = 5
    amount = 32

    self.list_dx, self.list_dy = calc_circle_pints(center, radius, amount)

    glEnable(GL_BLEND)
    glEnable(GL_LINE_SMOOTH)
    glBegin(GL_TRIANGLE_FAN)

    for x, y in zip(self.list_dx, self.list_dy):
        glVertex2f(x, y)

    glEnd()
    glDisable(GL_BLEND)
    glDisable(GL_LINE_SMOOTH)


def calc_circle_pints(center, radius, amount):
    list_dx = []
    list_dy = []

    x_offset, y_offset = center
    angle = math.radians(360 / amount)
    for i in range(amount):
        list_dx.append(math.cos(i*angle) * radius + x_offset)
        list_dy.append(math.sin(i*angle) * radius + y_offset)

    return list_dx, list_dy