import os
import bpy
import rna_keymap_ui
from bpy.props import *
from . utils.blender_ui import write_text, dpi_factor
from . utils.registers import hotkey_entry_item
from . utils.addons import addon_exists


def preference():
    name = addon_name()
    return bpy.context.user_preferences.addons[name].preferences


def addon_name():
    return os.path.basename(os.path.dirname(os.path.realpath(__file__)))

# Specific preference settings


def enable_experimental_tools():
    return preference().enable_experimental_tools


def enable_border_display():
    return preference().enable_border_display


def BC_border_color():
    return preference().BC_border_color


def BC_indicator_color1():
    return preference().BC_indicator_color1


def BC_indicator_color2():
    return preference().BC_indicator_color2


def enable_grid_display():
    return preference().enable_grid_display


def BC_grid_color():
    return preference().BC_grid_color

# properties


def BoxCutter_enable_help():
    return preference().BoxCutter_enable_help


def BoxCutter_display_shape():
    return preference().BoxCutter_display_shape


def BoxCutter_shape_moving():
    return preference().BoxCutter_shape_moving


def BoxCutter_enable_grid():
    return preference().BoxCutter_enable_grid


def BoxCutter_enable_hops():
    return preference().BoxCutter_enable_hops


def BoxCutter_circle_vert_count():
    return preference().BoxCutter_circle_vert_count


def BoxCutter_curve_vert_count():
    return preference().BoxCutter_curve_vert_count


def BoxCutter_enable_ngon():
    return preference().BoxCutter_enable_ngon


def BoxCutter_split_mesh():
    return preference().BoxCutter_split_mesh


def BoxCutter_show_wires():
    return preference().BoxCutter_show_wires


def BoxCutter_bevel_offset_step():
    return preference().BoxCutter_bevel_offset_step


def BoxCutter_grid_spaceing():
    return preference().BoxCutter_grid_spaceing


def BoxCutter_help_item():
    return preference().BoxCutter_help_item


def BoxCutter_set_cutter_shape():
    return preference().BoxCutter_set_cutter_shape


def BC_indicator_size():
    return preference().BC_indicator_size


def BC_indicator_x_position():
    return preference().BC_indicator_x_position


def BC_indicator_y_position():
    return preference().BC_indicator_y_position


def hud_y_offset():
    return preference().hud_y_offset


settings_tabs_items = [
    ("UI", "UI", ""),
    ("PROPERTIES", "Properties", ""),
    ("INFO", "Info", ""),
    ("KEYMAP", "Keymap", ""),
    ("LINKS", "Links / Help", "")]

help_items = [
    ("MAIN", "Main", ""),
    ("SHAPE", "Shape", ""),
    ("SHAPE_EDIT", "Shape edit", ""),
    ("BOX", "Box", ""),
    ("CIRCLE", "Circle", ""),
    ("NGON", "Ngon", ""),
    ("VROLL", "View Roll", ""),
    ("NONE", "None", "")]

cursor_usage = [
    ("TO", "To", ""),
    ("FROM", "From", "")]

cutter_shape = [
    ("BOX", "Box", ""),
    ("NGON", "Ngon", ""),
    ("CIRCLE", "Circle", "")]

bc_mode = [
    ("CUTTER", "Cutter", ""),
    ("MESHMAKER", "MeshMaker", "")]

mirror_type = [
    ("3D", "Origin", ""),
    ("2D", "Cursor", "")]

grid_type = [
    ("DEFAULT", "Default", ""),
    ("CUSTOM", "Custom", "")]

axis_items = [
    ("X", "X", ""),
    ("Y", "Y", ""),
    ("Z", "Z", "")]


class BoxCutterPreferences(bpy.types.AddonPreferences):
    bl_idname = addon_name()

    # properties
    BoxCutter_enter_ortho = BoolProperty(
        name="Enter Ortho View",
        description="enter ortho view",
        default=False)

    BoxCutter_rotation_value = IntProperty(
        name="Rotation Angle",
        description="Rotation Angle",
        default=0)

    BoxCutter_rotation_angle = IntProperty(
        name="Rotation Angle",
        description="Rotation Angle",
        default=5, min=1, max=360)

    BC_help_button_corner = IntProperty(
        name="Help Button Corner",
        description="Corner for the help button. 0 = top left. 1 = top right. 2 = bottom left. 3 = bottom right",
        default=3, min=0, max=3)

    boxcutter_mirror_clump = BoolProperty(
        name="Clump Mirror",
        description="Clump mirror",
        default=False)

    BoxCutter_2_77_fix = BoolProperty(
        name="D pie fix for old versions(below 2.79) ",
        description="D pie fix for old versions",
        default=False)


    BoxCutter_set_cutter_shape = EnumProperty(name="", default="BOX",
                                              options={"SKIP_SAVE"}, items=cutter_shape)

    BoxCutter_set_bc_mode = EnumProperty(name="", default="CUTTER",
                                              options={"SKIP_SAVE"}, items=bc_mode)

    BoxCutter_set_cursor_usage = EnumProperty(name="", default="TO",
                                              options={"SKIP_SAVE"}, items=cursor_usage)

    BoxCutter_mirror_type = EnumProperty(name="", default="2D",
                                         options={"SKIP_SAVE"}, items=mirror_type)

    BoxCutter_type_of_grid = EnumProperty(name="", default="DEFAULT",
                                          options={"SKIP_SAVE"}, items=grid_type)

    BoxCutter_box_depth = FloatProperty(
        name="BC Box Depth",
        description="BC box depth",
        default=0.5, min=0)

    BoxCutter_cut_depth = FloatProperty(
        name="BC Cut Depth",
        description="BC box depth",
        default=0, min=0)

    BoxCutter_mirror_axis_x = BoolProperty(
        name="X Mirror",
        description="Enable x mirror",
        default=False)

    BoxCutter_mirror_axis_y = BoolProperty(
        name="Y Mirror",
        description="Enable y mirror",
        default=False)

    BoxCutter_mirror_axis_z = BoolProperty(
        name="Z Mirror",
        description="Enable z mirror",
        default=False)

    BoxCutter_allow3d_cutter = BoolProperty(
        name="Allow 3d Cutter",
        description="Allow 3d cutter",
        default=True)

    BoxCutter_enable_help = BoolProperty(
        name="Help Info",
        description="Enable help info",
        default=True)

    BoxCutter_display_shape = BoolProperty(
        name="Shape",
        description="Open shape panel",
        default=False)

    BoxCutter_shape_moving = BoolProperty(
        name="Moving Shape",
        description="Enable shape moving",
        default=False)

    BoxCutter_enable_grid = BoolProperty(
        name="Enable Grid ",
        description="Enable grid",
        default=False)

    BoxCutter_enable_hops = BoolProperty(
        name="Enable Hops",
        description="Enable hops for box cutter",
        default=False)

    BoxCutter_circle_vert_count = IntProperty(
        name="Circle Verts",
        description="Circle Verts Count",
        default=32, min=3, max=128)

    BoxCutter_curve_vert_count = IntProperty(
        name="Curve Verts",
        description="Curve Verts Count",
        default=6, min=1, max=32)

    BoxCutter_enable_ngon = BoolProperty(
        name="Enable Ngon",
        description="Enable ngons drawing option for box cutter",
        default=True)

    BoxCutter_enable_borders = BoolProperty(
        name="Enable B",
        description="Enable borders",
        default=False)

    grey_box_select = BoolProperty(
        name="Grey Box Select",
        description="Keep the object selected after inserting a grey object",
        default=False)

    BoxCutter_split_mesh = BoolProperty(name="Split Mesh",
                                        description="Separate the mesh after CSplit",
                                        default=True)

    BoxCutter_show_wires = BoolProperty(name="Show Wies",
                                        description="Show cutter wires",
                                        default=True)

    BoxCutter_bevel_segments = IntProperty(
        name="Bevel Segments",
        description="Value for bevel segments",
        default=6, min=1, max=20)

    BoxCutter_bevel_offset_step = FloatProperty(
        name="Bevel Offset Step",
        description="Bevel offset step",
        default=1, min=0)

    BoxCutter_snapping_angle = IntProperty(
        name="Angle Snapping",
        description=" value for angle snapping ",
        default=5, min=1, max=90)

    BoxCutter_grid_spaceing = IntProperty(
        name="Grid Spaceing",
        description="Value for grid spaceing ",
        default=200, min=1, max=1000000)

    BC_indicator_size = FloatProperty(
        name="BoxCutter Indicator Size",
        description="BoxCutter indicator size",
        default=2, min=0, max=100)

    BC_indicator_x_position = FloatProperty(
        name="BoxCutter Indicator X Position",
        description="BoxCutter Indicator X Position",
        default=-78)

    BC_indicator_y_position = FloatProperty(
        name="BoxCutter Indicator Y Position",
        description="BoxCutter Indicator Y Position",
        default=1)

    BC_start_axis = EnumProperty(
        name="Rotation Start Axis",
        description="The initial axis to use when rotating the cutter",
        items=axis_items,
        default="X")

    BC_help_button_border_distance = FloatProperty(
        name="BoxCutter Help Button Border Padding",
        description="Amount of padding between button and bottom right corner",
        default=15, min=0, max=50)

    BC_help_button_border_distance = FloatProperty(
        name="BoxCutter Help Button Border Padding",
        description="Amount of padding between button and bottom right corner",
        default=15, min=0, max=50)

    BC_border_size = IntProperty(
        name="BoxCutter Border width",
        description="Amount of padding between button and bottom right corner",
        default=2, min=0)

    BC_border_offset = IntProperty(
        name="BoxCutter Border offset",
        description="Amount of padding between button and bottom right corner",
        default=15, min=0)

    workflow = EnumProperty(name="Mode",
                            description="",
                            items=[('DESTRUCTIVE', "Destructive", ""),
                                   ('NONDESTRUCTIVE', "NonDestructive", "")],
                            default='DESTRUCTIVE')

    hud_y_offset = BoolProperty(name="Change Hud Text Y Offset", default=False)

    BoxCutter_help_item = EnumProperty(name="", default="MAIN",
                                       options={"SKIP_SAVE"}, items=help_items)

    tab = EnumProperty(name="Tab", items=settings_tabs_items)

    BC_use_mirror = BoolProperty(name="Use Mirror", default=True)

    BC_start_curve_from_line = BoolProperty(name="Start From Point", default=True)

    # Main Prefs Needed
    BC_switch = BoolProperty(name="Enable BC", default=True)

    enable_experimental_tools = BoolProperty(name="Enable Experimental Tools", default=False)

    enable_border_display = BoolProperty(name="Enable Border Display", default=True)

    enable_grid_display = BoolProperty(name="Enable Grid Display", default=True)

    BC_border_color = FloatVectorProperty(
            name="",
            default=(0.29, 0.52, 1.0, 0.53),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_indicator_color1 = FloatVectorProperty(
            name="",
            default=(0.29, 0.52, 1.0, 0.9),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_indicator_color2 = FloatVectorProperty(
            name="",
            default=(0.66, 0.98, 0.26, 0.5),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_indicator_grid_color = FloatVectorProperty(
            name="",
            default=(0.29, 0.52, 1.0, 0.9),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_grid_color = FloatVectorProperty(
            name="",
            default=(0.2, 0.2, 0.2, 0.5),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_indicator_color_off = FloatVectorProperty(
            name="",
            default=(0.448, 0.448, 0.448, 0.1),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_redbox_color = FloatVectorProperty(
            name="",
            default=(0.6, 0.2, 0.2, 0.5),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_yellowbox_color = FloatVectorProperty(
            name="",
            default=(0.62, 0.5, 0.2, 0.5),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_graybox_color = FloatVectorProperty(
            name="",
            default=(0.5, 0.5, 0.5, 0.5),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_bluebox_color = FloatVectorProperty(
            name="",
            default=(0.29, 0.52, 1.0, 0.53),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_purplebox_color = FloatVectorProperty(
            name="",
            default=(0.4, 0.2, 0.8, 0.5),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_greenbox_color = FloatVectorProperty(
            name="",
            default=(0.2, 0.4, 0.1, 0.5),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_curveline_color = FloatVectorProperty(
            name="",
            default=(0.2, 0.2, 0.5, 0.85),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_cursorline_color = FloatVectorProperty(
            name="",
            default=(0.1, 0.4, 0.1, 0.85),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_helpbutton_color = FloatVectorProperty(
            name="",
            default=(0.25, 0.25, 0.25, 0.75),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_helptext_color = FloatVectorProperty(
            name="",
            default=(1.0, 1.0, 1.0, 1.0),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BC_verts_color = FloatVectorProperty(
            name="",
            default=(0, 0, 0, 0.75),
            size=4,
            min=0, max=1,
            subtype='COLOR'
            )

    BoxCutter_verts_size = FloatProperty(
        name="Vert size",
        description="Vertex size",
        default=2, min=0.1)

    BoxCutter_display_text_size = IntProperty(
        name="Text size",
        description="Curve line width",
        default=10, min=1)

    BoxCutter_curveline_width = FloatProperty(
        name="Curve Line Width",
        description="Curve line width",
        default=1.5, min=0.1)

    BoxCutter_curvepoints_size = FloatProperty(
        name="Curve Point Size",
        description="Curve Point Size",
        default=8, min=0.1)

    def draw(self, context):
        layout = self.layout

        col = layout.column(align=True)
        row = col.row()
        row.prop(self, "tab", expand=True)

        box = col.box()

        if self.tab == "UI":
            self.draw_ui_tab(box)
        elif self.tab == "PROPERTIES":
            self.draw_properties_tab(box)
        elif self.tab == "INFO":
            self.draw_info_tab(box)
        elif self.tab == "KEYMAP":
            self.draw_keymap_tab(box)
        elif self.tab == "LINKS":
            self.draw_links_tab(box)

    def draw_ui_tab(self, layout):
        box = layout.box()

        row = box.row(align=True)
        row.prop(self, "enable_border_display", text="Enable Border Display")
        row.prop(self, "hud_y_offset", text="Hud Text Y Offset")

        row = box.row(align=True)
        row.enabled = self.enable_border_display
        row.prop(self, "BC_border_color", text="Border Display")

        row = box.row(align=True)
        row.enabled = self.enable_border_display
        row.prop(self, "BC_border_size", text="Border Width")
        row.prop(self, "BC_border_offset", text="Border Offset")

        row = box.row(align=True)
        row.prop(self, "BC_indicator_color1", text="Indicator ON Color (Desctructive)")
        row.prop(self, "BC_indicator_color2", text="Indicator ON Color (Non-Desctructive)")
        row.prop(self, "BC_indicator_color_off", text="Indicator OFF Color")

        row = box.row(align=True)
        row.prop(self, "BC_indicator_size", text="Indicator Size")
        row.prop(self, "BC_indicator_x_position", text="Indicator X Position")
        row.prop(self, "BC_indicator_y_position", text="Indicator Y Position")
        row = box.row(align=True)
        row.prop(self, "BC_help_button_border_distance", text="Help Button padding")
        row.prop(self, "BC_help_button_corner", text="Help Button corner")
        box.separator()

        row = box.row(align=True)
        row.prop(self, "BC_redbox_color", text="RED BOX")
        row = box.row(align=True)
        row.prop(self, "BC_yellowbox_color", text="YELLOW BOX")
        row = box.row(align=True)
        row.prop(self, "BC_graybox_color", text="GRAY BOX")
        row = box.row(align=True)
        row.prop(self, "BC_bluebox_color", text="BLUE BOX")
        row = box.row(align=True)
        row.prop(self, "BC_purplebox_color", text="PURPLE BOX")
        row = box.row(align=True)
        row.prop(self, "BC_greenbox_color", text="GREEN BOX")
        row = box.row(align=True)
        row.prop(self, "BC_curveline_color", text="Curve Line Color")
        row = box.row(align=True)
        row.prop(self, "BC_cursorline_color", text="Cursor Line Color")
        row = box.row(align=True)
        row.prop(self, "BC_helpbutton_color", text="Help Button Color")
        row = box.row(align=True)
        row.prop(self, "BC_helptext_color", text="Help Text Color")
        row = box.row(align=True)
        row.prop(self, "BC_verts_color", text="BC Verts Color")

        row = box.row(align=True)
        row.label("Start Axis")
        row.prop(self, "BC_start_axis", text="Start Axis", expand=True)

        box.separator()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_verts_size", text="BC Verts Size")
        row = box.row(align=True)
        row.prop(self, "BoxCutter_display_text_size", text="BC Display Text Size")

        box.separator()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_curveline_width", text="Curve Line Size")
        row.prop(self, "BoxCutter_curvepoints_size", text="Curve Point Size")
        box.separator()

        row = box.row(align=True)
        row.prop(self, "enable_grid_display", text="Enable Grid Display")

        row = box.row(align=True)
        row.enabled = self.enable_grid_display
        row.prop(self, "BC_grid_color", text="Grid Color")



##        row = box.row(align=True)
##        row.prop(self, "BC_help_x_position", text="Help Button X Position")
##        row.prop(self, "BC_help_y_position", text="Help Button Y Position")
##
##        row = box.row(align=True)
##        row.prop(self, "BC_help_x_position_uncollapsed", text="Open Help Button X Position")
##        row.prop(self, "BC_help_y_position_uncollapsed", text="Open Help Button Y Position")




    def draw_info_tab(self, layout):
        write_text(layout, info_text, width=bpy.context.region.width / dpi_factor() / 8)

    def draw_keymap_tab(self, layout):
        box = layout.box()
        split = box.split()
        col = split.column()
        col.label('Do not remove hotkeys, disable them instead.')
        col.separator()
        col.label('Hotkeys')

        col.separator()

        wm = bpy.context.window_manager
        kc = wm.keyconfigs.user

        col.label('start BC:')

        col.separator()
        km = kc.keymaps['3D View']
        kmi = hotkey_entry_item(km, 'boxcutter.draw_boolean_layout', 'none', 'none')
        if kmi:
            col.context_pointer_set("keymap", km)
            rna_keymap_ui.draw_kmi([], kc, km, kmi, col, 0)
        else:
            col.label("No hotkey entry found")
            col.label("restore hotkeys from interface tab")

        col.label('enable/disable BC:')

        col.separator()
        km = kc.keymaps['3D View']
        kmi = hotkey_entry_item(km, 'boxcutter.switch', 'none', 'none')
        if kmi:
            col.context_pointer_set("keymap", km)
            rna_keymap_ui.draw_kmi([], kc, km, kmi, col, 0)
        else:
            col.label("No hotkey entry found")
            col.label("restore hotkeys from interface tab")

    def draw_links_tab(self, layout):
        col = layout.column()
        for name, url in weblinks:
            col.operator("wm.url_open", text=name).url = url

    def draw_properties_tab(self, layout):
        row = layout.row()

        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_2_77_fix", text="Fix for D pie (versions below 2.79)")

        layout.separator()
        layout.separator()

        if addon_exists("HOps") or any(name in bpy.context.user_preferences.addons for name in ('HOps', 'HOps-master', 'HardOps', 'hops', 'hardops')):
            box = layout.box()
            row = box.row(align=True)
            row.prop(self, "BoxCutter_enable_hops", text="Hard Ops Connection")
        elif self.BoxCutter_enable_hops:
            self.BoxCutter_enable_hops = False
        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "grey_box_select", text="Keep grey objects selected")
        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_enable_help", text="Display Help In Box Cutter")
        row.prop(self, "BoxCutter_show_wires", text="Show Wire In Cut Mode")
        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "BC_start_curve_from_line", text="Start Curve From Line")
        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_enter_ortho", text="Jump to ortho view when drawing")
        box = layout.box()
        row = box.row(align=True)

        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_circle_vert_count", text="Circle Vert Count")
        row = box.row(align=True)
        row.prop(self, "BoxCutter_curve_vert_count", text="Curve Vert Count")
        # row = box.row(align=True)
        # row.prop(self, "BoxCutter_enable_ngon", text = "enable ngons drawing option for box cutter")
        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_bevel_segments", text="Bevel Segments")
        row = box.row(align=True)
        row.prop(self, "BoxCutter_bevel_offset_step", text="Bevel Offset Step Size")
        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_snapping_angle", text="Angle Snapping Amount")
        box = layout.box()
        row = box.row(align=True)
        row.prop(self, "BoxCutter_enable_grid", text="Snapping / (Shift + Z)")
        row = box.row(align=True)
        row.prop(self, "BoxCutter_grid_spaceing", text="Snapping Amount")

        box = layout.box()
        row = box.row(align=True)
        row.label("Debug options")
        row = box.row(align=True)
        row.prop(self, "boxcutter_mirror_clump", text="Clump Mirror")
        row = box.row(align=True)
        row.prop(self, "BC_use_mirror", text="Enable mirror")


info_text = """ Thank you for your support.  / Alt + W Starts Box Cutter! And Esc ends it.


Box Cutter V4 Internal
""".replace("\n", " ")

weblinks = [
    ("AR",                      "https://www.artstation.com/artist/adrianrutkowski"),
    ("MX",                      "https://www.artstation.com/artist/jerryperkins1447"),
    ("Intro Guide",             "https://masterxeon1001.com/2016/05/28/box-cutter-4-update-notes/"),
]
