import bpy
from .. preferences import preference


class BoxCutterPieMenu(bpy.types.Menu):
    bl_label = "Box Cutter Pie Menu"
    bl_idname = "boxcutter_pie_menu"

    def check(self, context):
        return True

    def draw(self, context):
        layout = self.layout
        active_object = context.active_object

        if active_object is None:
            self.draw_object_mode_pie(context, layout)
        elif active_object.mode == "OBJECT":
            self.draw_object_mode_pie(context, layout)
        elif active_object.mode == "EDIT":
            self.draw_edit_mode_pie(context, layout)

    def draw_without_active_object_pie(self, layout):
        pie = self.layout.menu_pie()

        # left
        pie.separator()
        # right
        pie.separator()
        # bot
        pie.separator()
        # top
        pie.separator()
        # top L
        pie.separator()
        # top R
        pie.operator("boxcutter.draw_boolean_layout", text="Bool Layout")
        # bot L
        pie.operator("boxcutter.draw_boolean_layout", text="Bool Layout")
        # bot R
        pie.separator()

    def draw_edit_mode_pie(self, context, layout):
        pie = self.layout.menu_pie()

        # left
        pie.operator("boxcutter.set_circle_shape",  text="Circle")
        # right
        self.drawpie_edit_mode(layout)
        # bot
        pie.operator("boxcutter.set_ngon_shape", text="Ngon")
        # top
        self.drawpie_options(context, layout)
        # top L
        pie.separator()
        # top R
        pie.separator()
        # bot L
        pie.operator("boxcutter.set_box_shape", text="Box")
        # bot R
        pie.separator()

    def draw_object_mode_pie(self, context, layout):
        pie = self.layout.menu_pie()

        # left
        pie.operator("boxcutter.set_circle_shape",  text="Circle")
        # right
        self.drawpie_method(layout)
        # bot
        pie.operator("boxcutter.set_ngon_shape", text="Ngon")
        # top
        self.drawpie_options(context, layout)
        # top L
        pie.separator()
        # top R
        pie.separator()
        # bot L
        pie.operator("boxcutter.set_box_shape", text="Box")
        # bot R
        pie.separator()

    def drawpie_options(self, context, layout):

        pie = self.layout.menu_pie()

        col = pie.column(align=True)

        # colrow = col.row(align=True)
        col.prop(preference(), "BoxCutter_enable_help", text="Enable Help Info")

        col.separator()
        col.prop(preference(), "BoxCutter_circle_vert_count", text="Circle Resolution", slider=True)
        col.prop(preference(), "BoxCutter_curve_vert_count", text="Curve Resolution", slider=True)
        col.prop(preference(), "BoxCutter_bevel_segments", text="Bevel Segments", slider=True)
        col.prop(preference(), "BoxCutter_rotation_angle", text="Rotation Angle", slider=True)

        col.separator()

        colrow = col.row(align=True)
        colrow.prop(preference(), "BoxCutter_enable_grid", text="Grid")
        if preference().BoxCutter_enable_grid:
            colrow = col.row(align=True)
            colrow.prop(preference(), "BoxCutter_type_of_grid", text="Grid Type", expand=True)
            if preference().BoxCutter_type_of_grid == "CUSTOM":
                colrow = col.row(align=True)
                colrow.prop(preference(), "BoxCutter_grid_spaceing", text="Spacing")
            elif preference().BoxCutter_type_of_grid == "DEFAULT":
                colrow = col.row(align=True)
                colrow.prop(context.space_data, "grid_scale", text="Scale")
                colrow.prop(context.space_data, "grid_subdivisions", text="Subdivisions")

        col.separator()
        # col = row.column(align=True)
        # col.prop(preference(), "BoxCutter_mirror_type", text = "")
        if not preference().BoxCutter_2_77_fix:
            colrow = col.row(align=True)
            colrow.prop(preference(), "BoxCutter_mirror_type", text="Mirror From", expand=True)
            colrow = col.row(align=True)
            colrow.label("Mirror:")
            colrow.prop(preference(), "BoxCutter_mirror_axis_x", text="X", toggle=True)
            colrow.prop(preference(), "BoxCutter_mirror_axis_y", text="Y", toggle=True)

    def drawpie_method(self, layout):
        pie = self.layout.menu_pie()

        col = pie.column(align=True)

        # colrow = col.row(align=True)
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.label("Cutters:")
        col.prop(preference(), "BoxCutter_allow3d_cutter", text="Allow 3d Cutter")
        col.separator()
        colrow = col.row(align=True)
        colrow.prop(preference(), "workflow", text="Worflow", expand=True)
        col.prop(preference(), "grey_box_select", text="Grey Box Select")

        col.separator()
        col.label("Cursor:")
        colrow = col.row(align=True)
        colrow.prop(preference(), "BoxCutter_box_depth", text="Gray Box Size", expand=True)
        colrow = col.row(align=True)
        colrow.prop(preference(), "BoxCutter_cut_depth", text="Cut Depth", expand=True)

        # col.separator()
        # col.label("Borders:")
        # col.prop(preference(), "BoxCutter_enable_borders", text="Enable borders")

    def drawpie_edit_mode(self, layout):

        pie = self.layout.menu_pie()

        col = pie.column(align=True)

        # colrow = col.row(align=True)
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()
        col.separator()

        col.separator()
        col.label("Cutters:")
        col.prop(preference(), "grey_box_select", text="Grey Box Select")

        col.separator()
        col.label("Cursor:")
        colrow = col.row(align=True)
        colrow.prop(preference(), "BoxCutter_box_depth", text="Gray Box Size", expand=True)
        colrow = col.row(align=True)
        colrow.prop(preference(), "BoxCutter_cut_depth", text="Cut Depth", expand=True)
