import bpy

from bpy_extras.view3d_utils import location_3d_to_region_2d, region_2d_to_location_3d, region_2d_to_origin_3d, region_2d_to_vector_3d
from mathutils import Vector
from mathutils.geometry import intersect_line_plane

def center_point(self, context):
    region = context.region
    rv3d = region3d(self, context)
    coord = Vector((0, 0, 0, 0))
    point = location_3d_to_region_2d(region, rv3d, coord)

    return point

def point_to_2d(self, context, X, Y, Z):
    region = context.region
    rv3d = region3d(self, context)
    coord = Vector((X, Y, Z))
    point = location_3d_to_region_2d(region, rv3d, coord)

    return point

def cursor_2d_location(self, context):

    region = context.region
    rv3d = region3d(self, context)
    coord = context_cursor(context)
    point = location_3d_to_region_2d(region, rv3d, coord)

    return point

def context_cursor(context):
    scene = context.scene
    space = context.space_data

    cursor = (space if space and space.type == 'VIEW_3D'
              else scene).cursor_location

    return cursor

def set_context_cursor(context, cursor):
    scene = context.scene
    space = context.space_data

    (space if space and space.type == 'VIEW_3D' else scene).cursor_location = cursor

def bbox_center(context):
    object = bpy.context.object
    local_bbox_center = 0.125 * sum((Vector(b) for b in object.bound_box), Vector())
    global_bbox_center = object.matrix_world * local_bbox_center

    return global_bbox_center

def region3d(self, context):
    if not context.space_data.region_quadviews:
        rv3d = bpy.context.space_data.region_3d
    else:
        # verify area
        if context.area.type != 'VIEW_3D' or context.space_data.type != 'VIEW_3D':
            return
        i = -1
        for region in context.area.regions:
            if region.type == 'WINDOW':
                i += 1
                if context.region.id == region.id:
                    break
        else:
            return
        rv3d = context.space_data.region_quadviews[i]

    return rv3d

def point_in_rect(p, r):
    return ((p[0] >= r.x) and (p[0] < r.x + r.width) and (p[1] >= r.y) and (p[1] < r.y + r.height))

def rv3d_from_region(area, region):
    if (area.type != 'VIEW_3D') or (region.type != 'WINDOW'): return None

    space_data = area.spaces.active
    try:
        quadviews = space_data.region_quadviews
    except AttributeError:
        quadviews = None # old API

    if not quadviews: return space_data.region_3d

    x_id = 0
    y_id = 0
    for r in area.regions:
        if (r.type == 'WINDOW') and (r != region):
            if r.x < region.x: x_id = 1
            if r.y < region.y: y_id = 1

    # 0: bottom left (Front Ortho)
    # 1: top left (Top Ortho)
    # 2: bottom right (Right Ortho)
    # 3: top right (User Persp)
    return quadviews[y_id | (x_id << 1)]

# areas can't overlap, but regions can

def ui_contexts_under_coord(x, y, window=None):
    point = int(x), int(y)
    if not window: window = bpy.context.window
    screen = window.screen
    scene = screen.scene
    tool_settings = scene.tool_settings
    for area in screen.areas:
        if point_in_rect(point, area):
            space_data = area.spaces.active
            for region in area.regions:
                if point_in_rect(point, region):
                    yield dict(window=window, screen=screen,
                               area=area, space_data=space_data, region=region,
                               region_data=rv3d_from_region(area, region),
                               scene=scene, tool_settings=tool_settings)
            break

def region_exists(r):
    wm = bpy.context.window_manager
    for window in wm.windows:
        for area in window.screen.areas:
            for region in area.regions:
                if region == r: return True
    return False

def mouse_origin3d(context, event):
    mouse = event.mouse_region_x, event.mouse_region_y
    return region_2d_to_origin_3d(context.region, context.region_data, mouse)

def mouse_vector3d(context, event):
    mouse = event.mouse_region_x, event.mouse_region_y
    return region_2d_to_vector_3d(context.region, context.region_data, mouse)

def mouse_intersect3d(context, event, location, normal):
    mouse_origin = mouse_origin3d(context, event)
    return intersect_line_plane(mouse_origin, mouse_origin + mouse_vector3d(context, event), location, normal)

def location3d_to_point2d(context, location):
    return location_3d_to_region_2d(context.region, context.region_data, location)

def point2d_to_location3d(context, mouse, location):
    return region_2d_to_location_3d(context.region, context.region_data, mouse, location)

#TODO: needs methods for object? and scene
class ray_cast:

    def __new__(self, op, context, event):
        return self.cast(op, context, event)

    def cast(op, context, event):
        ray_object = None

        data = context.active_object.to_mesh(scene=context.scene, apply_modifiers=True, settings='PREVIEW')
        obj = bpy.data.objects.new('BC-duplicate', data)
        obj.matrix_world = context.active_object.matrix_world
        context.scene.objects.link(obj)
        context.scene.update()
        op.active_object = context.active_object
        context.scene.objects.active = obj
        bpy.ops.object.transform_apply(scale=True)

        success, hit_location, normal, face_index = ray_cast.search(context, event, obj, obj.matrix_world)

        if success:
            location_world = obj.matrix_world * hit_location
            length_squared = (location_world - mouse_origin3d(context, event)).length_squared

            if ray_object is None or length_squared < ray_length_squared:
                ray_length_squared = length_squared
                ray_object = obj

        if ray_object:
            ray_object.select = True
            context.scene.objects.active = op.active_object

            return ray_object, ray_object.matrix_world * hit_location, ray_object.matrix_world.to_3x3() * normal, face_index, ray_object.matrix_world * ray_object.data.polygons[face_index].center
        else:
            context.scene.objects.active = op.active_object
            try:
                bpy.data.meshes.remove(obj.data)
                bpy.data.objects.remove(obj, True)
            except: pass
            return None, None, None, None, None

    def search(context, event, obj, matrix):
        ray_origin = matrix.inverted() * mouse_origin3d(context, event)
        ray_target = matrix.inverted() * (mouse_origin3d(context, event) + mouse_vector3d(context, event))
        ray_direction = ray_target - ray_origin

        return obj.ray_cast(ray_origin, ray_direction)
