import bpy
import math


def set_viewport_ortho(context, mode="ORTHO"):
    for area in bpy.context.screen.areas:
        if area.type == 'VIEW_3D':
            area.spaces[0].region_3d.view_perspective = mode
            break


def view_orientation(view_quat):
    r = lambda x: round(x, 3)
    view_rot = view_quat.to_euler()

    orientation_dict = {(0.0, 0.0, 0.0): 'TOP',
                        (r(math.pi), 0.0, 0.0): 'BOTTOM',
                        (r(math.pi/2), 0.0, 0.0): 'FRONT',
                        (r(math.pi/2), 0.0, r(math.pi)): 'BACK',
                        (r(math.pi/2), 0.0, r(-math.pi/2)): 'LEFT',
                        (r(math.pi/2), 0.0, r(math.pi/2)): 'RIGHT'}

    return orientation_dict.get(tuple(map(r, view_rot)), 'UNDEFINED')


def set_3d_cursor_location(x, y, z):

    bpy.context.space_data.cursor_location[0] = x
    bpy.context.space_data.cursor_location[1] = y
    bpy.context.space_data.cursor_location[2] = z


def save_3d_cursor_location():

    cursor_x = bpy.context.space_data.cursor_location[0]
    cursor_y = bpy.context.space_data.cursor_location[1]
    cursor_z = bpy.context.space_data.cursor_location[2]

    return cursor_x, cursor_y, cursor_z


def rotate(origin, point, angle):
    ox, oy = origin
    px, py = point

    qx = ox + math.cos(angle) * (px - ox) - math.sin(angle) * (py - oy)
    qy = oy + math.sin(angle) * (px - ox) + math.cos(angle) * (py - oy)
    return qx, qy


def scale(origin, point, axis):
    ox, oy = origin
    px, py = point

    if axis == "X":
        px = (px-ox)*(-1)+ox
    elif axis == "Y":
        py = (py-oy)*-(1)+oy
    elif axis == "XY":
        px = (px-ox)*(-1)+ox
        py = (py-oy)*-(1)+oy

    return px, py


def centroid(points):
    x = [p[0] for p in points]
    y = [p[1] for p in points]
    centroid = (sum(x) / len(points), sum(y) / len(points))

    return centroid


def calculate_angle(point_a, point_b):
    ang_a = math.atan2(*point_a[::-1])
    ang_b = math.atan2(*point_b[::-1])

    return math.degrees((ang_a - ang_b) % (2 * math.pi))


def ccw(A, B, C):
    return (C.y-A.y)*(B.x-A.x) > (B.y-A.y)*(C.x-A.x)


def intersect(A, B, C, D):
    # return if line segments AB and CD intersect
    return ccw(A, C, D) != ccw(B, C, D) and ccw(A, B, C) != ccw(A, B, D)
